@@ -27,7 +27,7 @@ use Config::IniFiles;
 #inherits Lemonldap::NG::Common::Conf::Backends::SOAP
 #inherits Lemonldap::NG::Common::Conf::Backends::LDAP
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.9';
 our $msg     = '';
 our $iniObj;
 
@@ -57,7 +57,7 @@ BEGIN {
 # @return New Lemonldap::NG::Common::Conf object
 sub new {
     my $class = shift;
-    my $self = bless {}, $class;
+    my $self  = bless {}, $class;
     if ( ref( $_[0] ) ) {
         %$self = %{ $_[0] };
     }
@@ -87,7 +87,7 @@ sub new {
           unless $self->{type} =~ /^Lemonldap::/;
         eval "require $self->{type}";
         if ($@) {
-            $msg .= "Error: Unknown package $self->{type}.\n";
+            $msg .= "Error: failed to load $self->{type}: \n $@";
             return 0;
         }
         return 0 unless $self->prereq;
@@ -141,7 +141,7 @@ sub saveConf {
     if ( $self->{refLocalStorage} ) {
         $self->setDefault($conf);
         $self->compactConf($conf);
-        $self->setLocalConf($conf);
+        eval { Lemonldap::NG::Handler::Main->reload() };
     }
 
     return ( $self->unlock() ? $tmp : UNKNOWN_ERROR );
@@ -258,14 +258,6 @@ sub setDefault {
         $conf->{$k} //= $defaultValues->{$k};
     }
 
-    # Convert old option useXForwardedForIP into trustedProxies
-    if ( defined $conf->{useXForwardedForIP}
-        and $conf->{useXForwardedForIP} == 1 )
-    {
-        $conf->{trustedProxies} = '*';
-        delete $conf->{useXForwardedForIP};
-    }
-
     # Some parameters expect key name (example), not variable ($example)
     if ( defined $conf->{whatToTrace} ) {
         $conf->{whatToTrace} =~ s/^\$//;
@@ -312,7 +304,8 @@ sub getLocalConf {
         $cfg = Config::IniFiles->new( -file => $file, -allowcontinue => 1 );
 
         unless ( defined $cfg ) {
-            $msg .= "Local config error: " . @Config::IniFiles::errors . "\n";
+            $msg .= "Local config error: "
+              . ( join "\n", @Config::IniFiles::errors ) . "\n";
             return $r;
         }
 
@@ -342,7 +335,7 @@ sub getLocalConf {
             {
                 eval "\$r->{$_} = $r->{$_}";
                 if ($@) {
-                    $msg .= "Warning: error in file $file: $@.\n";
+                    $msg .= "Warn: error in file $file: $@.\n";
                     return $r;
                 }
             }
@@ -364,7 +357,7 @@ sub getLocalConf {
         if ( $r->{$_} =~ /^[{\[].*[}\]]$/ || $r->{$_} =~ /^sub\s*{.*}$/ ) {
             eval "\$r->{$_} = $r->{$_}";
             if ($@) {
-                $msg .= "Warning: error in file $file: $@.\n";
+                $msg .= "Warn: error in file $file: $@.\n";
                 return $r;
             }
         }
@@ -405,47 +398,63 @@ sub getDBConf {
     return $conf;
 }
 
+sub _launch {
+    my $self = shift;
+    my $sub  = shift;
+    my @res;
+    eval {
+        local $SIG{ALRM} = sub { die "TIMEOUT\n" };
+        eval {
+            alarm( $self->{confTimeout} || 10 );
+            @res = &{ $self->{type} . "::$sub" }( $self, @_ );
+        };
+        alarm 0;
+        die $@ if $@;
+    };
+    $msg .= $@ if $@;
+    return wantarray ? (@res) : $res[0];
+}
+
 ## @method boolean prereq()
 # Call prereq() from the $self->{type} package.
 # @return True if succeed
 sub prereq {
-    return &{ $_[0]->{type} . '::prereq' }(@_);
+    return shift->_launch( 'prereq', @_ );
 }
 
 ## @method @ available()
 # Call available() from the $self->{type} package.
 # @return list of available configuration numbers
 sub available {
-    return &{ $_[0]->{type} . '::available' }(@_);
+    return shift->_launch( 'available', @_ );
 }
 
 ## @method int lastCfg()
 # Call lastCfg() from the $self->{type} package.
 # @return Number of the last configuration available
 sub lastCfg {
-    my $result = &{ $_[0]->{type} . '::lastCfg' }(@_) || "0";
-    return $result;
+    return shift->_launch( 'lastCfg', @_ ) || 0;
 }
 
 ## @method boolean lock()
 # Call lock() from the $self->{type} package.
 # @return True if succeed
 sub lock {
-    return &{ $_[0]->{type} . '::lock' }(@_);
+    return shift->_launch( 'lock', @_ );
 }
 
 ## @method boolean isLocked()
 # Call isLocked() from the $self->{type} package.
 # @return True if database is locked
 sub isLocked {
-    return &{ $_[0]->{type} . '::isLocked' }(@_);
+    return shift->_launch( 'isLocked', @_ );
 }
 
 ## @method boolean unlock()
 # Call unlock() from the $self->{type} package.
 # @return True if succeed
 sub unlock {
-    return &{ $_[0]->{type} . '::unlock' }(@_);
+    return shift->_launch( 'unlock', @_ );
 }
 
 ## @method int store(hashRef conf)
@@ -453,14 +462,14 @@ sub unlock {
 # @param $conf Lemondlap configuration serialized
 # @return Number of new configuration stored if succeed, 0 else.
 sub store {
-    return &{ $_[0]->{type} . '::store' }(@_);
+    return shift->_launch( 'store', @_ );
 }
 
 ## @method load(int cfgNum, arrayRef fields)
 # Call load() from the $self->{type} package.
 # @return Lemonldap::NG Configuration hashRef if succeed, 0 else.
 sub load {
-    return &{ $_[0]->{type} . '::load' }(@_);
+    return shift->_launch( 'load', @_ );
 }
 
 ## @method boolean delete(int cfgNum)
@@ -471,7 +480,7 @@ sub delete {
     my ( $self, $c ) = @_;
     my @a = $self->available();
     if ( grep( /^$c$/, @a ) ) {
-        return &{ $self->{type} . '::delete' }( $self, $c );
+        return $self->_launch( 'delete', $c );
     }
     else {
         return 0;
@@ -479,7 +488,7 @@ sub delete {
 }
 
 sub logError {
-    return &{ $_[0]->{type} . '::logError' }(@_);
+    return shift->_launch( 'logError', @_ );
 }
 
 1;
