@@ -1,12 +1,12 @@
 package Lemonldap::NG::Common::PSGI;
 
-use 5.10.0;
+use strict;
 use Mouse;
 use JSON;
 use Lemonldap::NG::Common::PSGI::Constants;
 use Lemonldap::NG::Common::PSGI::Request;
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.10';
 
 our $_json = JSON->new->allow_nonref;
 
@@ -17,6 +17,7 @@ has languages    => ( is => 'rw', isa     => 'Str', default => 'en' );
 has logLevel     => ( is => 'rw', isa     => 'Str', default => 'info' );
 has portal       => ( is => 'rw', isa     => 'Str' );
 has staticPrefix => ( is => 'rw', isa     => 'Str' );
+has instanceName => ( is => 'rw', isa     => 'Str', default => '' );
 has templateDir  => ( is => 'rw', isa     => 'Str|ArrayRef' );
 has links        => ( is => 'rw', isa     => 'ArrayRef' );
 has menuLinks    => ( is => 'rw', isa     => 'ArrayRef' );
@@ -34,17 +35,17 @@ sub init {
     foreach my $k ( keys %$args ) {
         $self->{$k} = $args->{$k} unless ( $k eq 'logger' );
     }
-    unless ( $self->logger and $self->userLogger ) {
+    unless ( ref( $self->logger ) and ref( $self->userLogger ) ) {
         my $logger =
              $args->{logger}
           || $ENV{LLNG_DEFAULTLOGGER}
           || 'Lemonldap::NG::Common::Logger::Std';
-        unless ( $self->logger ) {
+        unless ( ref $self->logger ) {
             eval "require $logger";
             die $@ if ($@);
             $self->logger( $logger->new($self) );
         }
-        unless ( $self->userLogger ) {
+        unless ( ref $self->userLogger ) {
             $logger = $ENV{LLNG_USERLOGGER} || $args->{userLogger} || $logger;
             eval "require $logger";
             die $@ if ($@);
@@ -124,7 +125,23 @@ sub sendJSONresponse {
     $args{headers} ||= [ $req->spliceHdrs ];
     my $type = 'application/json; charset=utf-8';
     if ( ref $j ) {
-        eval { $j = $_json->encode($j); };
+        eval {
+            if ( $args{pretty} ) {
+
+                # This avoids changing the settings of the $_json reference
+                $j = to_json(
+                    $j,
+                    {
+                        allow_nonref => 1,
+                        pretty       => 1,
+                        canonical    => 1
+                    }
+                );
+            }
+            else {
+                $j = $_json->encode($j);
+            }
+        };
         return $self->sendError( $req, $@ ) if ($@);
     }
     return [ $args{code}, [ 'Content-Type' => $type, @{ $args{headers} } ],
@@ -174,6 +191,8 @@ sub sendError {
             : $code == 400 ? 'Bad request'
             :                'Error'
         );
+
+        # TODO: this should probably use a template instead
         my $s = "<html><head><title>$title</title>
 <style>
 body{background:#000;color:#fff;padding:10px 50px;font-family:sans-serif;}a{text-decoration:none;color:#fff;}h1{text-align:center;}
@@ -185,18 +204,25 @@ body{background:#000;color:#fff;padding:10px 50px;font-family:sans-serif;}a{text
 <center><a href=\"https://lemonldap-ng.org\">LemonLDAP::NG</a></center>
 </body>
 </html>";
-        return [
-            $code,
-            [
-                'Content-Type'   => 'text/html; charset=utf-8',
-                'Content-Length' => length($s),
-                $req->spliceHdrs,
-            ],
-            [$s]
-        ];
+        return $self->sendRawHtml( $req, $s, code => $code );
     }
 }
 
+sub sendRawHtml {
+    my ( $self, $req, $s, %args ) = @_;
+    my $code    = $args{code}    || 200;
+    my $headers = $args{headers} || [ $req->spliceHdrs ];
+    return [
+        $code,
+        [
+            'Content-Type'   => 'text/html; charset=utf-8',
+            'Content-Length' => length($s),
+            @{$headers},
+        ],
+        [$s]
+    ];
+}
+
 sub abort {
     my ( $self, $err ) = @_;
     eval { $self->logger->error($err) };
@@ -271,6 +297,7 @@ sub sendHtml {
         # TODO: warn if STATICPREFIX does not end with '/'
         $htpl->param(
             STATIC_PREFIX => $sp,
+            INSTANCE_NAME => $self->instanceName,
             SCRIPTNAME    => $sc,
             ( $self->can('tplParams') ? ( $self->tplParams($req) ) : () ),
             (
