@@ -1,7 +1,7 @@
 # Main running methods file
 package Lemonldap::NG::Handler::Main::Run;
 
-our $VERSION = '2.0.7';
+our $VERSION = '2.0.10';
 
 package Lemonldap::NG::Handler::Main;
 
@@ -105,6 +105,7 @@ sub checkType {
 sub run {
     my ( $class, $req, $rule, $protection ) = @_;
     my ( $id, $session );
+    my $vhost = $class->resolveAlias($req);
 
     return $class->DECLINED unless ( $class->is_initial_req($req) );
 
@@ -149,9 +150,41 @@ sub run {
 
         # ACCOUNTING (1. Inform web server)
         $class->set_user( $req, $session->{ $class->tsv->{whatToTrace} } );
-        $class->set_custom( $req, $session->{ $class->tsv->{customToTrace} } )
-          if $class->tsv->{customToTrace}
-          and $session->{ $class->tsv->{customToTrace} };
+
+        my $custom;
+        $custom = $session->{ $class->tsv->{customToTrace} }
+          if (  $class->tsv->{customToTrace}
+            and $session->{ $class->tsv->{customToTrace} } );
+        if ( $class->tsv->{accessToTrace}->{$vhost} ) {
+            my ( $function, @params ) = split /\s*,\s*/,
+              $class->tsv->{accessToTrace}->{$vhost};
+            if ( $function =~ qr/^(?:\w+(?:::\w+)*(?:\s+\w+(?:::\w+)*)*)?$/ ) {
+                my $c = eval {
+                    no strict 'refs';
+                    &{$function}( {
+                            req     => $req,
+                            vhost   => $vhost,
+                            session => $session,
+                            custom  => $custom,
+                            params  => \@params
+                        }
+                    );
+                };
+                if ($@) {
+                    $class->logger->error(
+                        "Failed to overwrite customToTrace: $@");
+                }
+                else {
+                    $class->logger->debug("Overwrite customToTrace with: $c");
+                    $custom = $c;
+                }
+            }
+            else {
+                $class->logger->error(
+                    "accessToTrace: Bad custom function name");
+            }
+        }
+        $class->set_custom( $req, $custom ) if $custom;
 
         # AUTHORIZATION
         return ( $class->forbidden( $req, $session ), $session )
@@ -195,6 +228,10 @@ sub run {
     elsif ( $protection == $class->MAYSKIP
         and $class->grant( $req, $session, $uri, $cond ) eq '999_SKIP' )
     {
+        $class->logger->debug("Access control skipped");
+        $class->updateStatus( $req, 'SKIP' );
+        $class->hideCookie($req);
+        $class->cleanHeaders($req);
         return $class->OK;
     }
 
@@ -265,17 +302,14 @@ sub checkMaintenanceMode {
     return 0;
 }
 
-## @rmethod boolean grant(string uri, string cond)
-# Grant or refuse client using compiled regexp and functions
+## @rmethod int getLevel(string uri, string $vhost)
+# Return required authentication level for this URI
+# default to vhost authentication level
 # @param $uri URI
-# @param $cond optional Function granting access
-# @return True if the user is granted to access to the current URL
-sub grant {
-    my ( $class, $req, $session, $uri, $cond, $vhost ) = @_;
+# @param $vhost vhost name, default to current request
+sub getLevel {
+    my ( $class, $req, $uri, $vhost ) = @_;
     my $level;
-
-    return $cond->( $req, $session ) if ($cond);
-
     $vhost ||= $class->resolveAlias($req);
 
     # Using URL authentification level if exists
@@ -290,13 +324,33 @@ sub grant {
             last;
         }
     }
-    $level
-      ? $class->logger->debug(
-        'Found AuthnLevel=' . $level . ' for "' . "$vhost$uri" . '"' )
-      : $class->logger->debug("No URL authentication level found...");
+    if ($level) {
+        $class->logger->debug(
+            'Found AuthnLevel=' . $level . ' for "' . "$vhost$uri" . '"' );
+        return $level;
+    }
+    else {
+        $class->logger->debug("No URL authentication level found...");
+        return $class->tsv->{authnLevel}->{$vhost};
+    }
+}
+
+## @rmethod boolean grant(string uri, string cond)
+# Grant or refuse client using compiled regexp and functions
+# @param $uri URI
+# @param $cond optional Function granting access
+# @return True if the user is granted to access to the current URL
+sub grant {
+    my ( $class, $req, $session, $uri, $cond, $vhost ) = @_;
+
+    return $cond->( $req, $session ) if ($cond);
+
+    $vhost ||= $class->resolveAlias($req);
+
+    my $level = $class->getLevel( $req, $uri );
 
     # Using VH authentification level if exists
-    if ( $level ||= $class->tsv->{authnLevel}->{$vhost} ) {
+    if ($level) {
         if ( $session->{authenticationLevel} < $level ) {
             $class->logger->debug(
                 "User authentication level = $session->{authenticationLevel}");
@@ -339,7 +393,10 @@ sub grant {
 # @return Constant $class->FORBIDDEN
 sub forbidden {
     my ( $class, $req, $session, $vhost ) = @_;
-    my $uri = $req->{env}->{REQUEST_URI};
+    my $uri    = $req->{env}->{REQUEST_URI};
+    my $portal = $class->tsv->{portal}->();
+    $portal = ( $portal =~ m#^https?://([^/]*).*# )[0];
+    $portal =~ s/:\d+$//;
     $vhost ||= $class->resolveAlias($req);
 
     if ( $session->{_logout} ) {
@@ -362,11 +419,12 @@ sub forbidden {
         $session->{ $class->tsv->{whatToTrace} } );
 
     # Redirect or Forbidden?
-    if ( $class->tsv->{useRedirectOnForbidden} ) {
-        $class->logger->debug("Use redirect for forbidden access");
+    if ( $class->tsv->{useRedirectOnForbidden} && $vhost ne $portal ) {
+        $class->logger->debug("Use redirect for forbidden access ");
         return $class->goToError( $req, $uri, 403 );
     }
     else {
+        $class->logger->debug("Self protected Portal URL") if $vhost eq $portal;
         $class->logger->debug("Return forbidden access");
         return $class->FORBIDDEN;
     }
@@ -398,7 +456,7 @@ sub hideCookie {
 sub encodeUrl {
     my ( $class, $req, $url ) = @_;
     $url = $class->_buildUrl( $req, $url ) if ( $url !~ m#^https?://# );
-    return encode_base64( $url, '' );
+    return uri_escape( encode_base64( $url, '' ) );
 }
 
 ## @rmethod protected int goToPortal(string url, string arg)
@@ -437,15 +495,16 @@ sub goToError {
 # @return Value of the cookie if found, 0 else
 sub fetchId {
     my ( $class, $req ) = @_;
-    my $t                 = $req->{env}->{HTTP_COOKIE} or return 0;
-    my $vhost             = $class->resolveAlias($req);
+    my $t     = $req->{env}->{HTTP_COOKIE} or return 0;
+    my $vhost = $class->resolveAlias($req);
+    $class->logger->debug("VH $vhost is HTTPS")
+      if $class->_isHttps( $req, $vhost );
     my $lookForHttpCookie = ( $class->tsv->{securedCookie} =~ /^(2|3)$/
           and not $class->_isHttps( $req, $vhost ) );
     my $cn = $class->tsv->{cookieName};
-    my $value =
-      $lookForHttpCookie
-      ? ( $t =~ /${cn}http=([^,; ]+)/o ? $1 : 0 )
-      : ( $t =~ /$cn=([^,; ]+)/o       ? $1 : 0 );
+    my $value = $lookForHttpCookie    # Avoid prefix and bad cookie name (#2417)
+      ? ( $t =~ /(?<![-.~])\b${cn}http=([^,; ]+)/o ? $1 : 0 )
+      : ( $t =~ /(?<![-.~])\b$cn=([^,; ]+)/o       ? $1 : 0 );
 
     if ( $value && $lookForHttpCookie && $class->tsv->{securedCookie} == 3 ) {
         $value = $class->tsv->{cipher}->decryptHex( $value, "http" );
@@ -480,6 +539,9 @@ sub retrieveSession {
         $class->logger->debug("Get session $id from Handler internal cache");
         return $class->data;
     }
+    else {
+        $class->data( {} );
+    }
 
     # 2. Get the session from cache or backend
     my $session = $req->data->{session} = (
@@ -573,11 +635,8 @@ sub retrieveSession {
 # Returns the port on which this vhost is accessed
 # @param $s VHost name
 # @return PORT
-
 sub _getPort {
-
     my ( $class, $req, $vhost ) = @_;
-
     if ( defined $class->tsv->{port}->{$vhost}
         and ( $class->tsv->{port}->{$vhost} > 0 ) )
     {
@@ -590,19 +649,18 @@ sub _getPort {
             return $class->tsv->{port}->{_};
         }
         else {
-            return $req->{env}->{SERVER_PORT};
+            return $req->port;
         }
     }
 }
-## @cmethod private boot _isHttps(string s)
-# Returns whether this VHost should he accessed
+
+## @cmethod private bool _isHttps(string s)
+# Returns whether this VHost should be accessed
 # via HTTPS
 # @param $s VHost name
-# @return RUE if the vhost should be accessed over HTTPS
+# @return TRUE if the vhost should be accessed over HTTPS
 sub _isHttps {
-
     my ( $class, $req, $vhost ) = @_;
-
     if ( defined $class->tsv->{https}->{$vhost}
         and ( $class->tsv->{https}->{$vhost} > -1 ) )
     {
@@ -615,7 +673,7 @@ sub _isHttps {
             return $class->tsv->{https}->{_};
         }
         else {
-            return ( ( uc( $req->{env}->{HTTPS} ) || "OFF" ) eq "ON" );
+            return $req->secure;
         }
     }
 }
@@ -710,6 +768,7 @@ sub cleanHeaders {
     my ( $class, $req ) = @_;
     my $vhost = $class->resolveAlias($req);
     if ( defined( $class->tsv->{headerList}->{$vhost} ) ) {
+        $class->logger->debug("Remove headers relative to $vhost");
         $class->unset_header_in( $req,
             @{ $class->tsv->{headerList}->{$vhost} } );
     }
@@ -725,10 +784,8 @@ sub resolveAlias {
     return $class->tsv->{vhostAlias}->{$vhost}
       if ( $class->tsv->{vhostAlias}->{$vhost} );
     return $vhost if ( $class->tsv->{defaultCondition}->{$vhost} );
-    my $v = $vhost;
-    while ( $v =~ s/[\w\-]+/\*/ ) {
-        return $v if ( $class->tsv->{defaultCondition}->{$v} );
-        $v =~ s/^\*\.*//;
+    foreach ( @{ $class->tsv->{vhostReg} } ) {
+        return $_->[1] if $vhost =~ $_->[0];
     }
     return $vhost;
 }
@@ -766,21 +823,21 @@ sub abort {
 sub localUnlog {
     my ( $class, $req, $id ) = @_;
     $class->logger->debug('Local handler logout');
+
+    # Delete thread data
+    delete $req->data->{session};
+    $class->data( {} );
     if ( $id //= $class->fetchId($req) ) {
 
-        # Delete thread data
-        if (    $class->data->{_session_id}
-            and $id eq $class->data->{_session_id} )
-        {
-            $class->data( {} );
-        }
-        delete $req->data->{session};
-
         # Delete local cache
-        if (    $class->tsv->{refLocalStorage}
-            and $class->tsv->{refLocalStorage}->get($id) )
-        {
-            $class->tsv->{refLocalStorage}->remove($id);
+        if ( $class->tsv->{sessionCacheModule} ) {
+            my $module  = $class->tsv->{sessionCacheModule};
+            my $options = $class->tsv->{sessionCacheOptions};
+            eval "use $module;";
+            my $cache = $module->new($options);
+            if ( $cache->get($id) ) {
+                $cache->remove($id);
+            }
         }
     }
 }
