@@ -326,6 +326,12 @@ use constant HANDLERSECTION  => "handler";
 use constant MANAGERSECTION  => "manager";
 use constant SESSIONSEXPLORERSECTION => "sessionsExplorer";
 use constant APPLYSECTION            => "apply";
+
+# Default configuration backend
+use constant DEFAULTCONFBACKEND => "File";
+use constant DEFAULTCONFBACKENDOPTIONS => (
+    dirName => '/usr/local/lemonldap-ng/data/conf',
+);
 $confConstants
 our \@sessionTypes = ( '$sessionTypes' );
 
@@ -348,6 +354,8 @@ our %EXPORT_TAGS = (
           MANAGERSECTION
           SESSIONSEXPLORERSECTION
           APPLYSECTION
+          DEFAULTCONFBACKEND
+          DEFAULTCONFBACKENDOPTIONS
           NO
           \$hashParameters
           \@sessionTypes
@@ -425,7 +433,8 @@ sub buildZeroConf {
     open( F, '>', $self->firstLmConfFile ) or die($!);
     my $tmp = Lemonldap::NG::Manager::Conf::Zero::zeroConf(
         '__DNSDOMAIN__',   '__SESSIONDIR__',
-        '__PSESSIONDIR__', '__NOTIFICATIONDIR__'
+        '__PSESSIONDIR__', '__NOTIFICATIONDIR__',
+        '__CACHEDIR__'
     );
     $tmp->{cfgNum} = 1;
     print F $jsonEnc->encode($tmp);
@@ -456,15 +465,16 @@ our \$VERSION = '$Lemonldap::NG::Manager::Build::Attributes::VERSION';
 use constant HANDLER => 'Lemonldap::NG::Handler::PSGI::Main';
 use constant {
 EOF
-    for my $pe ( sort { $portalConstants{$a} <=> $portalConstants{$b} }
-        keys %portalConstants )
+    for my $pe (
+        sort { $portalConstants{$a} <=> $portalConstants{$b} }
+        keys %portalConstants
+      )
     {
         my $str = $portalConstants{$pe};
         $content .= "    $pe => $str,\n";
     }
 
-    my $exports = join ", ",
-      map  { "'$_'" }
+    my $exports = join ", ", map { "'$_'" }
       sort { $portalConstants{$a} <=> $portalConstants{$b} }
       keys %portalConstants;
 
@@ -615,10 +625,10 @@ sub scanTree {
             # Get data type and build tree
             #
             # Types : PerlModule bool boolOrExpr catAndAppList file hostname int
-            # keyTextContainer lmAttrOrMacro longtext openidServerList pcre
-            # rulesContainer samlAssertion samlAttributeContainer samlService
-            # select text trool url virtualHostContainer word
-            # password
+            # keyTextContainer lmAttrOrMacro longtext openidServerList
+            # oidcAttributeContainer pcre rulesContainer samlAssertion
+            # samlAttributeContainer samlService select text trool url
+            # virtualHostContainer word password
 
             if ( $leaf =~ s/^\*// ) {
                 push @angularScopeVars, [ $leaf, "$path._nodes[$ord]" ];
@@ -650,12 +660,18 @@ sub scanTree {
                     my $type = $attr->{type};
                     $type =~ s/Container//;
                     foreach my $k ( sort keys( %{ $attr->{default} } ) ) {
+                        # Special handling for oidcAttribute
+                        my $default = $attr->{default}->{$k};
+                        if ( $attr->{type} eq 'oidcAttributeContainer' ) {
+                            $default = [ $default, "string", "auto" ];
+                        }
+
                         push @{ $jleaf->{default} },
                           {
                             id    => "$prefix$leaf/$k",
                             title => $k,
                             type  => $type,
-                            data  => $attr->{default}->{$k},
+                            data  => $default,
                             (
                                 $type eq 'rule'
                                 ? ( re => $k )
