@@ -14,7 +14,7 @@
 
 package Lemonldap::NG::Manager::Build::CTrees;
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.8';
 
 sub cTrees {
     return {
@@ -27,10 +27,10 @@ sub cTrees {
                 help  => 'configvhost.html#options',
                 form  => 'simpleInputContainer',
                 nodes => [
-                    'vhostPort',        'vhostHttps',
-                    'vhostMaintenance', 'vhostAliases',
-                    'vhostType',        'vhostAuthnLevel',
-                    'vhostServiceTokenTTL'
+                    'vhostPort',          'vhostHttps',
+                    'vhostMaintenance',   'vhostAliases',
+                    'vhostAccessToTrace', 'vhostType',
+                    'vhostAuthnLevel',    'vhostServiceTokenTTL'
                 ],
             },
         ],
@@ -52,6 +52,7 @@ sub cTrees {
                 title => "samlIDPMetaDataOptionsSignature",
                 form  => 'simpleInputContainer',
                 nodes => [
+                    "samlIDPMetaDataOptionsSignatureMethod",
                     "samlIDPMetaDataOptionsSignSSOMessage",
                     "samlIDPMetaDataOptionsCheckSSOMessageSignature",
                     "samlIDPMetaDataOptionsSignSLOMessage",
@@ -122,10 +123,11 @@ sub cTrees {
                         title => "samlSPMetaDataOptionsSignature",
                         form  => 'simpleInputContainer',
                         nodes => [
+                            "samlSPMetaDataOptionsSignatureMethod",
                             "samlSPMetaDataOptionsSignSSOMessage",
                             "samlSPMetaDataOptionsCheckSSOMessageSignature",
                             "samlSPMetaDataOptionsSignSLOMessage",
-                            "samlSPMetaDataOptionsCheckSLOMessageSignature"
+                            "samlSPMetaDataOptionsCheckSLOMessageSignature",
                         ]
                     },
                     {
@@ -134,6 +136,7 @@ sub cTrees {
                         nodes => [
                             "samlSPMetaDataOptionsEncryptionMode",
                             "samlSPMetaDataOptionsEnableIDPInitiatedURL",
+                            "samlSPMetaDataOptionsAuthnLevel",
                             "samlSPMetaDataOptionsRule",
                         ]
                     }
@@ -191,27 +194,51 @@ sub cTrees {
             'oidcRPMetaDataOptionsExtraClaims',
             {
                 title => 'oidcRPMetaDataOptions',
+                help  => 'idpopenidconnect.html#options',
                 nodes => [ {
-                        title => 'oidcRPMetaDataOptionsAuthentication',
+                        title => 'oidcRPMetaDataOptionsBasic',
                         form  => 'simpleInputContainer',
                         nodes => [
                             'oidcRPMetaDataOptionsClientID',
                             'oidcRPMetaDataOptionsClientSecret',
                             'oidcRPMetaDataOptionsPublic',
+                            'oidcRPMetaDataOptionsRedirectUris',
+                        ]
+                    },
+                    {
+                        title => 'oidcRPMetaDataOptionsAdvanced',
+                        form  => 'simpleInputContainer',
+                        nodes => [
+                            'oidcRPMetaDataOptionsBypassConsent',
+                            'oidcRPMetaDataOptionsUserIDAttr',
+                            'oidcRPMetaDataOptionsIDTokenForceClaims',
+                            'oidcRPMetaDataOptionsAdditionalAudiences',
+                            'oidcRPMetaDataOptionsRefreshToken',
+                        ]
+                    },
+                    {
+                        title => 'security',
+                        form  => 'simpleInputContainer',
+                        nodes => [
+                            'oidcRPMetaDataOptionsIDTokenSignAlg',
                             'oidcRPMetaDataOptionsRequirePKCE',
+                            'oidcRPMetaDataOptionsAllowOffline',
+                            'oidcRPMetaDataOptionsAllowPasswordGrant',
+                            'oidcRPMetaDataOptionsAllowClientCredentialsGrant',
+                            'oidcRPMetaDataOptionsAuthnLevel',
+                            'oidcRPMetaDataOptionsRule',
+                        ]
+                    },
+                    {
+                        title => 'oidcRPMetaDataOptionsTimeouts',
+                        form  => 'simpleInputContainer',
+                        nodes => [
+                            'oidcRPMetaDataOptionsAuthorizationCodeExpiration',
+                            'oidcRPMetaDataOptionsIDTokenExpiration',
+                            'oidcRPMetaDataOptionsAccessTokenExpiration',
+                            'oidcRPMetaDataOptionsOfflineSessionExpiration',
                         ]
                     },
-                    'oidcRPMetaDataOptionsUserIDAttr',
-                    'oidcRPMetaDataOptionsIDTokenSignAlg',
-                    'oidcRPMetaDataOptionsIDTokenExpiration',
-                    'oidcRPMetaDataOptionsIDTokenForceClaims',
-                    'oidcRPMetaDataOptionsAccessTokenExpiration',
-                    'oidcRPMetaDataOptionsAuthorizationCodeExpiration',
-                    'oidcRPMetaDataOptionsAllowOffline',
-                    'oidcRPMetaDataOptionsRefreshToken',
-                    'oidcRPMetaDataOptionsOfflineSessionExpiration',
-                    'oidcRPMetaDataOptionsRedirectUris',
-                    'oidcRPMetaDataOptionsBypassConsent',
                     {
                         title => 'logout',
                         form  => 'simpleInputContainer',
@@ -222,7 +249,6 @@ sub cTrees {
                             'oidcRPMetaDataOptionsLogoutSessionRequired',
                         ]
                     },
-                    'oidcRPMetaDataOptionsRule',
                 ]
             },
             'oidcRPMetaDataMacros',
@@ -265,6 +291,7 @@ sub cTrees {
                 nodes => [
                     'casAppMetaDataOptionsService',
                     'casAppMetaDataOptionsUserAttribute',
+                    'casAppMetaDataOptionsAuthnLevel',
                     'casAppMetaDataOptionsRule'
                 ]
             },
