@@ -6,20 +6,25 @@
 # Read methods are inherited from Lemonldap::NG::Common::Conf::RESTServer
 package Lemonldap::NG::Manager::Conf;
 
-use 5.10.0;
+use strict;
 use utf8;
 use Mouse;
 use Lemonldap::NG::Common::Conf::Constants;
 use Lemonldap::NG::Common::UserAgent;
+use Lemonldap::NG::Common::EmailTransport;
 use Crypt::OpenSSL::RSA;
 use Convert::PEM;
+use Digest::MD5 qw(md5_base64);
+
 use URI::URL;
+use Net::SSLeay;
 
-use feature 'state';
+extends qw(
+  Lemonldap::NG::Manager::Plugin
+  Lemonldap::NG::Common::Conf::RESTServer
+);
 
-extends 'Lemonldap::NG::Common::Conf::RESTServer';
-
-our $VERSION = '2.0.7';
+our $VERSION = '2.0.10';
 
 #############################
 # I. INITIALIZATION METHODS #
@@ -27,36 +32,9 @@ our $VERSION = '2.0.7';
 
 use constant defaultRoute => 'manager.html';
 
-has ua       => ( is => 'rw' );
-has diffRule => ( is => 'rw', default => sub { 0 } );
-has brwRule  => ( is => 'rw', default => sub { 0 } );
-
-sub addRoutes {
+sub init {
     my ( $self, $conf ) = @_;
     $self->ua( Lemonldap::NG::Common::UserAgent->new($conf) );
-    my $hd = "Lemonldap::NG::Handler::PSGI::Main";
-
-    # Parse Diff activation rule
-    $self->logger->debug(
-        "Diff activation rule -> " . ( $self->{viewerAllowDiff} // 0 ) );
-    my $rule = $hd->buildSub( $hd->substitute( $self->{viewerAllowDiff} ) );
-    unless ($rule) {
-        $self->error(
-            "Bad Diff activation rule -> " . $hd->tsv->{jail}->error );
-        return 0;
-    }
-    $self->diffRule($rule);
-
-    # Parse Browser activation rule
-    $self->logger->debug(
-        "Browser activation rule -> " . ( $self->{viewerAllowBrowser} // 0 ) );
-    $rule = $hd->buildSub( $hd->substitute( $self->{viewerAllowBrowser} ) );
-    unless ($rule) {
-        $self->error(
-            "Bad Browser activation rule -> " . $hd->tsv->{jail}->error );
-        return 0;
-    }
-    $self->brwRule($rule);
 
     # HTML template
     $self->addRoute( 'manager.html', undef, ['GET'] )
@@ -82,9 +60,11 @@ sub addRoutes {
       # New key and conf save
       ->addRoute(
         confs => {
-            newRSAKey => 'newRSAKey',
-            raw       => 'newRawConf',
-            '*'       => 'newConf'
+            newRSAKey      => 'newRSAKey',
+            newCertificate => 'newCertificate',
+            sendTestMail   => 'sendTestMail',
+            raw            => 'newRawConf',
+            '*'            => 'newConf'
         },
         ['POST']
       )
@@ -95,6 +75,32 @@ sub addRoutes {
 
       # Url loader
       ->addRoute( 'prx', undef, ['POST'] );
+    return 1;
+}
+
+# 35 - New Certificate on demand
+#      --------------------------
+
+##@method public PSGI-JSON-response newRSAKey($req)
+# Return a hashref containing private and public keys
+# The posted data must contain a JSON object containing
+# {"password":"newpassword"}
+#
+#@param $req Lemonldap::NG::Common::PSGI::Request object
+#@return PSGI JSON response
+sub newCertificate {
+    my ( $self, $req, @others ) = @_;
+    return $self->sendError( $req, 'There is no subkey for "newCertificate"',
+        400 )
+      if (@others);
+    my $query = $req->jsonBodyToObj;
+
+    my ( $private, $cert ) = $self->_generateX509( $query->{password} );
+    my $keys = {
+        'private' => $private,
+        'public'  => $cert,
+    };
+    return $self->sendJSONresponse( $req, $keys );
 }
 
 # 35 - New RSA key pair on demand
@@ -111,11 +117,13 @@ sub newRSAKey {
     my ( $self, $req, @others ) = @_;
     return $self->sendError( $req, 'There is no subkey for "newRSAKey"', 400 )
       if (@others);
+    my $rsa = Crypt::OpenSSL::RSA->generate_key(2048);
+
     my $query = $req->jsonBodyToObj;
-    my $rsa   = Crypt::OpenSSL::RSA->generate_key(2048);
     my $keys  = {
         'private' => $rsa->get_private_key_string(),
         'public'  => $rsa->get_public_key_x509_string(),
+        'hash' => md5_base64($rsa->get_public_key_string()),
     };
     if ( $query->{password} ) {
         my $pem = Convert::PEM->new(
@@ -142,6 +150,127 @@ sub newRSAKey {
     return $self->sendJSONresponse( $req, $keys );
 }
 
+# This function does the dirty X509 work,
+# mostly copied from IO::Socket::SSL::Utils
+# and adapter to work on old platforms (CentOS7)
+
+sub _generateX509 {
+    my ( $self, $password ) = @_;
+    Net::SSLeay::SSLeay_add_ssl_algorithms();
+    my $conf = $self->confAcc->getConf();
+
+    # Generate 2048 bits RSA key
+    my $key = Net::SSLeay::EVP_PKEY_new();
+    Net::SSLeay::EVP_PKEY_assign_RSA( $key,
+        Net::SSLeay::RSA_generate_key( 2048, 0x10001 ) );
+
+    my $cert = Net::SSLeay::X509_new();
+
+    # Serial
+    Net::SSLeay::ASN1_INTEGER_set(
+        Net::SSLeay::X509_get_serialNumber($cert),
+        rand( 2**32 ),
+    );
+
+    # Version
+    Net::SSLeay::X509_set_version( $cert, 2 );
+
+    # Make it last 20 years
+    Net::SSLeay::ASN1_TIME_set( Net::SSLeay::X509_get_notBefore($cert),
+        time() );
+    Net::SSLeay::ASN1_TIME_set( Net::SSLeay::X509_get_notAfter($cert),
+        time() + 20 * 365 * 86400 );
+
+    # set subject
+    my $portal_uri  = new URI::URL( $conf->{portal} || "http://localhost" );
+    my $portal_host = $portal_uri->host;
+    my $subj_e      = Net::SSLeay::X509_get_subject_name($cert);
+    my $subj        = { commonName => $portal_host, };
+
+    while ( my ( $k, $v ) = each %$subj ) {
+
+        # Not everything we get is nice - try with MBSTRING_UTF8 first and if it
+        # fails try V_ASN1_T61STRING and finally V_ASN1_OCTET_STRING
+        Net::SSLeay::X509_NAME_add_entry_by_txt( $subj_e, $k, 0x1000, $v, -1,
+            0 )
+          or
+          Net::SSLeay::X509_NAME_add_entry_by_txt( $subj_e, $k, 20, $v, -1, 0 )
+          or
+          Net::SSLeay::X509_NAME_add_entry_by_txt( $subj_e, $k, 4, $v, -1, 0 )
+          or croak( "failed to add entry for $k - "
+              . Net::SSLeay::ERR_error_string( Net::SSLeay::ERR_get_error() ) );
+    }
+
+    # Set to self-sign
+    Net::SSLeay::X509_set_pubkey( $cert, $key );
+    Net::SSLeay::X509_set_issuer_name( $cert,
+        Net::SSLeay::X509_get_subject_name($cert) );
+
+    # Sign with default alg
+    Net::SSLeay::X509_sign( $cert, $key, 0 );
+
+    my $strCert = Net::SSLeay::PEM_get_string_X509($cert);
+    my $strPrivate;
+    if ($password) {
+        $strPrivate = Net::SSLeay::PEM_get_string_PrivateKey( $key, $password );
+    }
+    else {
+        $strPrivate = Net::SSLeay::PEM_get_string_PrivateKey($key);
+    }
+
+    # Free OpenSSL objects
+    Net::SSLeay::X509_free($cert);
+    Net::SSLeay::EVP_PKEY_free($key);
+
+    return ( $strPrivate, $strCert );
+}
+
+#      Sending a test Email
+#      --------------------
+
+##@method public PSGI-JSON-response sendTestMail($req)
+# Sends a test email to the provided address
+# The posted data must contain a JSON object containing
+# {"dest":"target@example.com"}
+#
+#@param $req Lemonldap::NG::Common::PSGI::Request object
+#@return PSGI JSON response
+sub sendTestMail {
+    my ( $self, $req, @others ) = @_;
+    return $self->sendError( $req, 'There is no subkey for "sendTestMail"',
+        400 )
+      if (@others);
+    my $dest = $req->jsonBodyToObj->{dest};
+    unless ($dest) {
+        $self->logger->debug("Missing dest parameter for sending test mail");
+        return $self->sendJSONresponse(
+            $req,
+            {
+                success => \0,
+                error   => "You must provide an email address"
+            }
+        );
+    }
+    my $conf = $self->confAcc->getConf();
+
+    # Try to send test Email
+    $self->logger->info("Sending test email to $dest");
+    eval {
+        Lemonldap::NG::Common::EmailTransport::sendTestMail( $conf, $dest );
+    };
+    my $error   = $@;
+    my $success = ( $error ? 0 : 1 );
+
+    $self->logger->debug("Email was sent") unless $error;
+    return $self->sendJSONresponse(
+        $req,
+        {
+            success => \$success,
+            ( $error ? ( error => $error ) : () )
+        }
+    );
+}
+
 # 36 - URL File loader
 #      ---------------
 
@@ -178,16 +307,15 @@ sub prx {
 # IV. Upload methods #
 ######################
 
-# In this section, 4 methods:
+# In this section, 3 methods:
 #  - getConfByNum: override SUPER method to be able to use Zero
-#  - newConf()
+#  - newConf(), load a new configuration and invokes reloadUrls
 #  - newRawConf(): restore a saved conf
-#  - applyConf(): called by the 2 previous to prevent other servers that a new
-#                 configuration is available
 
 sub getConfByNum {
     my ( $self, $cfgNum, @args ) = @_;
-    unless ( %{ $self->currentConf }
+    unless ($self->currentConf
+        and %{ $self->currentConf }
         and $cfgNum == $self->currentConf->{cfgNum} )
     {
         my $tmp;
@@ -252,7 +380,7 @@ sub newConf {
 
     if ( $cfgNum ne $req->params('cfgNum') ) { $parser->confChanged(1); }
 
-    my $res = { result => $parser->check($self) };
+    my $res = { result => $parser->check( $self->p ) };
 
     # "message" fields: note that words enclosed by "__" (__word__) will be
     # translated
@@ -268,7 +396,7 @@ sub newConf {
         }
     }
     if ( $res->{result} ) {
-        if ( $self->{demoMode} ) {
+        if ( $self->p->{demoMode} ) {
             $res->{message} = '__demoModeOn__';
         }
         else {
@@ -279,7 +407,7 @@ sub newConf {
               unless ( @{ $parser->{needConfirmation} } && !$args{force} );
             if ( $s > 0 ) {
                 $self->userLogger->notice(
-                    'User ' . $self->userId($req) . " has stored conf $s" );
+                    'User ' . $self->p->userId($req) . " has stored conf $s" );
                 $res->{result} = 1;
                 $res->{cfgNum} = $s;
                 if ( my $status = $self->applyConf( $parser->newConf ) ) {
@@ -291,7 +419,7 @@ sub newConf {
             else {
                 $self->userLogger->notice(
                     'Saving attempt rejected, asking for confirmation to '
-                      . $self->userId($req) );
+                      . $self->p->userId($req) );
                 $res->{result} = 0;
                 if ( $s == CONFIG_WAS_CHANGED ) {
                     $res->{needConfirm} = 1;
@@ -324,7 +452,7 @@ sub newRawConf {
     }
 
     my $res = {};
-    if ( $self->{demoMode} ) {
+    if ( $self->p->{demoMode} ) {
         $res->{message} = '__demoModeOn__';
     }
     else {
@@ -332,15 +460,16 @@ sub newRawConf {
         # chances to be equal to last config cfgNum
         my $s = $self->confAcc->saveConf( $new, force => 1 );
         if ( $s > 0 ) {
-            $self->userLogger->notice(
-                'User ' . $self->userId($req) . " has stored (raw) conf $s" );
+            $self->userLogger->notice( 'User '
+                  . $self->p->userId($req)
+                  . " has stored (raw) conf $s" );
             $res->{result} = 1;
             $res->{cfgNum} = $s;
         }
         else {
             $self->userLogger->notice(
                 'Raw saving attempt rejected, asking for confirmation to '
-                  . $self->userId($req) );
+                  . $self->p->userId($req) );
             $res->{result}      = 0;
             $res->{needConfirm} = 1 if ( $s == CONFIG_WAS_CHANGED );
             $res->{message} .= '__needConfirmation__';
@@ -349,64 +478,6 @@ sub newRawConf {
     return $self->sendJSONresponse( $req, $res );
 }
 
-## @method private applyConf()
-# Try to prevent other servers declared in `reloadUrls` that a new
-# configuration is available.
-#
-#@return reload status as boolean
-sub applyConf {
-    my ( $self, $newConf ) = @_;
-    my $status;
-
-    # 1 Apply conf locally
-    $self->api->checkConf();
-
-    # Get apply section values
-    my %reloadUrls =
-      %{ $self->confAcc->getLocalConf( APPLYSECTION, undef, 0 ) };
-    if ( !%reloadUrls && $newConf->{reloadUrls} ) {
-        %reloadUrls = %{ $newConf->{reloadUrls} };
-    }
-    return {} unless (%reloadUrls);
-
-    $self->ua->timeout( $newConf->{reloadTimeout} );
-
-    # Parse apply values
-    while ( my ( $host, $request ) = each %reloadUrls ) {
-        my $r = HTTP::Request->new( 'GET', "http://$host$request" );
-        if ( $request =~ /^https?:\/\/[^\/]+.*$/ ) {
-            my $url       = URI::URL->new($request);
-            my $targetUrl = $url->scheme . "://" . $host;
-            $targetUrl .= ":" . $url->port if defined( $url->port );
-            $targetUrl .= $url->full_path;
-            $r =
-              HTTP::Request->new( 'GET', $targetUrl,
-                HTTP::Headers->new( Host => $url->host ) );
-            if ( defined $url->userinfo
-                && $url->userinfo =~ /^([^:]+):(.*)$/ )
-            {
-                $r->authorization_basic( $1, $2 );
-            }
-        }
-
-        my $response = $self->ua->request($r);
-        if ( $response->code != 200 ) {
-            $status->{$host} =
-              "Error " . $response->code . " (" . $response->message . ")";
-            $self->logger->error( "Apply configuration for $host: error "
-                  . $response->code . " ("
-                  . $response->message
-                  . ")" );
-        }
-        else {
-            $status->{$host} = "OK";
-            $self->logger->notice("Apply configuration for $host: ok");
-        }
-    }
-
-    return $status;
-}
-
 sub diff {
     my ( $self, $req, @path ) = @_;
     return $self->sendError( $req, 'to many arguments in path info', 400 )
