@@ -8,23 +8,24 @@ use strict;
 use Mouse;
 use JSON qw(from_json to_json);
 use Lemonldap::NG::Portal::Main::Constants qw(
-  PE_BADOTP
-  PE_ERROR
-  PE_FORMEMPTY
   PE_OK
+  PE_ERROR
+  PE_BADOTP
+  PE_FORMEMPTY
   PE_SENDRESPONSE
 );
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.10';
 
-extends 'Lemonldap::NG::Portal::Main::SecondFactor',
-  'Lemonldap::NG::Common::TOTP';
+extends qw(
+  Lemonldap::NG::Portal::Main::SecondFactor
+  Lemonldap::NG::Common::TOTP
+);
 
 # INITIALIZATION
 
 has prefix => ( is => 'ro', default => 'totp' );
-
-has logo => ( is => 'rw', default => 'totp.png' );
+has logo   => ( is => 'rw', default => 'totp.png' );
 
 sub init {
     my ($self) = @_;
@@ -47,17 +48,21 @@ sub run {
     $self->logger->debug('Generate TOTP form');
 
     my $checkLogins = $req->param('checkLogins');
-    $self->logger->debug("TOTP checkLogins set") if ($checkLogins);
+    $self->logger->debug("TOTP: checkLogins set") if $checkLogins;
+
+    my $stayconnected = $req->param('stayconnected');
+    $self->logger->debug("TOTP: stayconnected set") if $stayconnected;
 
     # Prepare form
     my $tmp = $self->p->sendHtml(
         $req,
         'totp2fcheck',
         params => {
-            MAIN_LOGO   => $self->conf->{portalMainLogo},
-            SKIN        => $self->p->getSkin($req),
-            TOKEN       => $token,
-            CHECKLOGINS => $checkLogins
+            MAIN_LOGO     => $self->conf->{portalMainLogo},
+            SKIN          => $self->p->getSkin($req),
+            TOKEN         => $token,
+            CHECKLOGINS   => $checkLogins,
+            STAYCONNECTED => $stayconnected
         }
     );
     $self->logger->debug("Prepare TOTP 2F verification");
@@ -75,8 +80,7 @@ sub verify {
         return PE_FORMEMPTY;
     }
 
-    my $secret = '';
-    my $_2fDevices;
+    my ( $secret, $_2fDevices );
     if ( $session->{_2fDevices} ) {
         $self->logger->debug("Loading 2F Devices ...");
 
@@ -88,13 +92,10 @@ sub verify {
             return PE_ERROR;
         }
         $self->logger->debug("2F Device(s) found");
-        foreach (@$_2fDevices) {
-            $self->logger->debug("Reading TOTP secret if exists ...");
-            if ( $_->{type} eq 'TOTP' ) {
-                $secret = $_->{_secret};
-                last;
-            }
-        }
+        $self->logger->debug("Reading TOTP secret if exists...");
+
+        $secret = $_->{_secret}
+          foreach grep { $_->{type} eq 'TOTP' } @$_2fDevices;
     }
 
     unless ($secret) {
@@ -108,15 +109,15 @@ sub verify {
         $self->conf->{totp2fDigits},
         $secret, $code
     );
-    if ( $r == -1 ) { return PE_ERROR; }
-    elsif ($r) {
+    return PE_ERROR if ( $r == -1 );
+
+    if ($r) {
         $self->userLogger->info('TOTP succeed');
         return PE_OK;
     }
     else {
-        $self->userLogger->notice( 'Invalid TOTP for '
-              . $session->{ $self->conf->{whatToTrace} }
-              . ')' );
+        $self->userLogger->notice(
+            'Invalid TOTP for ' . $session->{ $self->conf->{whatToTrace} } );
         return PE_BADOTP;
     }
 }
