@@ -4,30 +4,29 @@
 # have registered their U2F key
 package Lemonldap::NG::Portal::2F::U2F;
 
-#use 5.16.0;
 use strict;
 use Mouse;
 use JSON qw(from_json to_json);
 use Lemonldap::NG::Portal::Main::Constants qw(
-  PE_BADCREDENTIALS
-  PE_ERROR
   PE_OK
-  PE_SENDRESPONSE
+  PE_ERROR
   PE_U2FFAILED
+  PE_SENDRESPONSE
+  PE_BADCREDENTIALS
 );
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.10';
 
-extends 'Lemonldap::NG::Portal::Main::SecondFactor',
-  'Lemonldap::NG::Portal::Lib::U2F';
+extends qw(
+  Lemonldap::NG::Portal::Main::SecondFactor
+  Lemonldap::NG::Portal::Lib::U2F
+);
 
 # INITIALIZATION
 
-has rule => ( is => 'rw' );
-
+has rule   => ( is => 'rw' );
 has prefix => ( is => 'ro', default => 'u' );
-
-has logo => ( is => 'rw', default => 'u2f.png' );
+has logo   => ( is => 'rw', default => 'u2f.png' );
 
 sub init {
     my ($self) = @_;
@@ -43,6 +42,7 @@ sub init {
     return 0
       unless ( $self->Lemonldap::NG::Portal::Main::SecondFactor::init()
         and $self->Lemonldap::NG::Portal::Lib::U2F::init() );
+
     1;
 }
 
@@ -53,7 +53,10 @@ sub run {
     my ( $self, $req, $token ) = @_;
 
     my $checkLogins = $req->param('checkLogins');
-    $self->logger->debug("U2F checkLogins set") if ($checkLogins);
+    $self->logger->debug("U2F: checkLogins set") if $checkLogins;
+
+    my $stayconnected = $req->param('stayconnected');
+    $self->logger->debug("U2F: stayconnected set") if $stayconnected;
 
     # Check if user is registered
     if ( my $res = $self->loadUser( $req, $req->sessionInfo ) ) {
@@ -71,12 +74,9 @@ sub run {
         }
 
         # Get registered keys
-        my @rk;
-        foreach ( @{ $req->data->{crypter} } ) {
-            push @rk,
-              { keyHandle => $_->{keyHandle}, version => $data->{version} };
-
-        }
+        my @rk =
+          map { { keyHandle => $_->{keyHandle}, version => $data->{version} } }
+          @{ $req->data->{crypter} };
 
         $self->ott->updateToken( $token, __ch => $data->{challenge} );
 
@@ -95,11 +95,12 @@ sub run {
             $req,
             'u2fcheck',
             params => {
-                MAIN_LOGO   => $self->conf->{portalMainLogo},
-                SKIN        => $self->p->getSkin($req),
-                DATA        => $data,
-                TOKEN       => $token,
-                CHECKLOGINS => $checkLogins
+                MAIN_LOGO     => $self->conf->{portalMainLogo},
+                SKIN          => $self->p->getSkin($req),
+                DATA          => $data,
+                TOKEN         => $token,
+                CHECKLOGINS   => $checkLogins,
+                STAYCONNECTED => $stayconnected
             }
         );
 
@@ -111,6 +112,7 @@ sub run {
 
 sub verify {
     my ( $self, $req, $session ) = @_;
+    my $crypter;
 
     # Check U2F signature
     if (    my $resp = $req->param('signature')
@@ -138,10 +140,9 @@ sub verify {
             $req->error(PE_ERROR);
             return $self->fail($req);
         }
-        my $crypter;
-        foreach ( @{ $req->data->{crypter} } ) {
-            $crypter = $_ if ( $_->{keyHandle} eq $data->{keyHandle} );
-        }
+        $crypter = $_
+          foreach grep { $_->{keyHandle} eq $data->{keyHandle} }
+          @{ $req->data->{crypter} };
         unless ($crypter) {
             $self->userLogger->error("Unregistered U2F key");
             $req->error(PE_BADCREDENTIALS);
@@ -247,9 +248,8 @@ sub loadUser {
                     'U2F error: ' . Crypt::U2F::Server::u2fclib_getError() );
             }
         }
-        unless (@crypters) {
-            return -1;
-        }
+        return -1 unless @crypters;
+
         $req->data->{crypter} = \@crypters;
         return 1;
     }
