@@ -3,12 +3,17 @@ package Lemonldap::NG::Portal::Lib::LDAP;
 use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Lib::Net::LDAP;
-use Lemonldap::NG::Portal::Main::Constants
-  qw(PE_OK PE_LDAPCONNECTFAILED PE_LDAPERROR PE_BADCREDENTIALS);
+use Lemonldap::NG::Portal::Main::Constants qw(
+  PE_OK
+  PE_LDAPERROR
+  PE_USERNOTFOUND
+  PE_BADCREDENTIALS
+  PE_LDAPCONNECTFAILED
+);
 
 extends 'Lemonldap::NG::Common::Module';
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.11';
 
 # PROPERTIES
 
@@ -67,6 +72,16 @@ has mailFilter => (
     builder => 'buildMailFilter',
 );
 
+has findUserFilter => (
+    is      => 'ro',
+    lazy    => 1,
+    builder => sub {
+        $_[0]->conf->{AuthLDAPFilter}
+          || $_[0]->conf->{LDAPFilter}
+          || '(&(uid=$user)(objectClass=inetOrgPerson))';
+    }
+);
+
 sub buildFilter {
     return $_[0]->_buildFilter( $_[0]->conf->{AuthLDAPFilter}
           || $_[0]->conf->{LDAPFilter}
@@ -115,10 +130,7 @@ sub getUser {
     my ( $self, $req, %args ) = @_;
 
     $self->validateLdap;
-
-    unless ( $self->ldap ) {
-        return PE_LDAPCONNECTFAILED;
-    }
+    return PE_LDAPCONNECTFAILED unless $self->ldap;
 
     $self->bind();
 
@@ -130,7 +142,7 @@ sub getUser {
             ? $self->mailFilter->($req)
             : $self->filter->($req)
         ),
-        defer => $self->conf->{ldapSearchDeref} || 'find',
+        deref => $self->conf->{ldapSearchDeref} || 'find',
         attrs => $self->attrs,
     );
     if ( $mesg->code() != 0 ) {
@@ -144,12 +156,75 @@ sub getUser {
         return PE_BADCREDENTIALS;
     }
     unless ( $req->data->{ldapentry} = $mesg->entry(0) ) {
-        $self->userLogger->warn("$req->{user} was not found in LDAP directory");
+        $self->userLogger->warn(
+                "$req->{user} was not found in LDAP directory ("
+              . $req->address
+              . ")" );
         eval { $self->p->_authentication->setSecurity($req) };
         return PE_BADCREDENTIALS;
     }
     $req->data->{dn} = $req->data->{ldapentry}->dn();
-    PE_OK;
+
+    return PE_OK;
+}
+
+sub findUser {
+    my ( $self, $req, %args ) = @_;
+    my $plugin =
+      $self->p->loadedModules->{"Lemonldap::NG::Portal::Plugins::FindUser"};
+    my ( $searching, $excluding ) = $plugin->retreiveFindUserParams($req);
+    eval { $self->p->_authentication->setSecurity($req) };
+    return PE_OK unless scalar @$searching;
+
+    $self->validateLdap;
+    return PE_LDAPCONNECTFAILED unless $self->ldap;
+
+    $self->findUserFilter =~ /\bobjectClass=(\w+)\b/;
+    my $filter   = "(&(objectClass=$1)";
+    my $wildcard = $self->conf->{findUserWildcard};
+    $self->logger->info("LDAP UserDB with wildcard ($wildcard)") if $wildcard;
+    foreach (@$searching) {
+        if ($wildcard) {
+            $_->{value} =~ s/\Q$wildcard\E+/*/g;
+        }
+        else {
+            $_->{value} =~ s/\Q*\E+//g;
+        }
+        $filter .= "($_->{key}=$_->{value})";
+    }
+    $filter .= "(!($_->{key}=$_->{value}))" foreach (@$excluding);
+    $filter .= ')';
+    $self->logger->debug("LDAP UserDB built filter: $filter");
+
+    $self->bind();
+    my $mesg = $self->ldap->search(
+        base   => $self->conf->{ldapBase},
+        scope  => 'sub',
+        filter => $filter,
+        deref  => $self->conf->{ldapSearchDeref} || 'find',
+        attrs  => $self->attrs,
+    );
+
+    if ( $mesg->code() != 0 ) {
+        $self->logger->error(
+            'LDAP Search error ' . $mesg->code . ": " . $mesg->error );
+        return PE_LDAPERROR;
+    }
+
+    $self->logger->debug(
+        'LDAP UserDB number of result(s): ' . $mesg->count() );
+    if ( $mesg->count() ) {
+        my $rank = int( rand( $mesg->count() ) );
+        $self->logger->debug("Demo UserDB random rank: $rank");
+        my $entry =
+          ( $mesg->entry($rank)->dn() =~ /\b(?:uid|sAMAccountName)=(\w+?)\b/ )
+          [0];
+        $self->userLogger->info("FindUser: LDAP UserDB returns $entry");
+        $req->data->{findUser} = $entry;
+        return PE_OK;
+    }
+
+    return PE_USERNOTFOUND;
 }
 
 # Validate LDAP connection before use
@@ -168,13 +243,14 @@ sub bind {
     my $self = shift;
 
     $self->validateLdap;
+    return undef unless $self->ldap;
 
-    return undef unless ( $self->ldap );
     my $msg = $self->ldap->bind(@_);
     if ( $msg->code ) {
         $self->logger->error( $msg->error );
         return undef;
     }
+
     return 1;
 }
 
