@@ -19,7 +19,7 @@ use Mouse;
 
 use Lemonldap::NG::Portal::Main::Constants qw(PE_OK PE_REDIRECT);
 
-our $VERSION = '2.0.7';
+our $VERSION = '2.0.11';
 
 # OpenID Connect standard claims
 use constant PROFILE => [
@@ -28,7 +28,7 @@ use constant PROFILE => [
 ];
 use constant EMAIL => [qw/email email_verified/];
 use constant ADDRESS =>
-  [qw/formatted street_address locality region postal_code/];
+  [qw/formatted street_address locality region postal_code country/];
 use constant PHONE => [qw/phone_number phone_number_verified/];
 
 # PROPERTIES
@@ -383,6 +383,10 @@ sub buildHybridAuthnResponse {
             ? ( token_type => 'bearer', access_token => $access_token )
             : ()
         ),
+        (
+            $id_token ? ( id_token => $id_token )
+            : ()
+        ),
         ( $expires_in    ? ( expires_in    => $expires_in )    : () ),
         ( $state         ? ( state         => $state )         : () ),
         ( $session_state ? ( session_state => $session_state ) : () )
@@ -639,10 +643,7 @@ sub decodeJSON {
     my $json_hash;
 
     eval { $json_hash = from_json( $json, { allow_nonref => 1 } ); };
-
-    if ($@) {
-        $json_hash->{error} = "parse_error";
-    }
+    $json_hash->{error} = "parse_error" if ($@);
 
     return $json_hash;
 }
@@ -1332,11 +1333,13 @@ sub buildUserInfoResponse {
     my ( $self, $req, $scope, $rp, $session ) = @_;
     my $userinfo_response = {};
 
-    my $user_id_attribute =
-      $self->conf->{oidcRPMetaDataOptions}->{$rp}
-      ->{oidcRPMetaDataOptionsUserIDAttr}
-      || $self->conf->{whatToTrace};
-    my $user_id = $session->data->{$user_id_attribute};
+    my $data = {
+        %{ $session->data },
+        _clientId => $self->oidcRPList->{$rp}->{oidcRPMetaDataOptionsClientID},
+        _clientConfKey => $rp,
+        _scope         => $scope,
+    };
+    my $user_id = $self->getUserIDForRP( $req, $rp, $data );
 
     $self->logger->debug("Found corresponding user: $user_id");
 
@@ -1349,38 +1352,138 @@ sub buildUserInfoResponse {
         my $list = $self->getAttributesListFromClaim( $rp, $claim );
         next unless $list;
         foreach my $attribute (@$list) {
-            my $session_key =
-              $self->conf->{oidcRPMetaDataExportedVars}->{$rp}->{$attribute};
+            my @attrConf = split /;/,
+              ( $self->conf->{oidcRPMetaDataExportedVars}->{$rp}->{$attribute}
+                  || "" );
+            my $session_key = $attrConf[0];
             if ($session_key) {
+                my $type  = $attrConf[1] || 'string';
+                my $array = $attrConf[2] || 'auto';
 
                 my $session_value;
 
                 # Lookup attribute in macros first
                 if ( $self->spMacros->{$rp}->{$session_key} ) {
-                    $session_value = $self->spMacros->{$rp}->{$session_key}
-                      ->( $req, $session->data );
+                    $session_value =
+                      $self->spMacros->{$rp}->{$session_key}->( $req, $data );
 
                     # If not found, search in session
                 }
                 else {
-                    $session_value = $session->data->{$session_key};
+                    $session_value = $data->{$session_key};
                 }
 
-                # Address is a JSON object
-                if ( $claim eq "address" ) {
-                    $userinfo_response->{address}->{$attribute} =
-                      $session_value;
-                }
-                else {
-                    $userinfo_response->{$attribute} = $session_value;
+                # Handle empty values, arrays, type, etc.
+                $session_value =
+                  $self->_formatValue( $session_value, $type, $array,
+                    $attribute, $req->user );
+
+             # From this point on, do NOT touch $session_value or you will break
+             # the variable's type.
+
+                # Only release claim if it has a value
+                if ( defined $session_value ) {
+
+                    # Address is a JSON object
+                    if ( $claim eq "address" ) {
+                        $userinfo_response->{address}->{$attribute} =
+                          $session_value;
+                    }
+                    else {
+                        $userinfo_response->{$attribute} = $session_value;
+                    }
                 }
             }
         }
     }
 
+    my $h = $self->p->processHook( $req, 'oidcGenerateUserInfoResponse',
+        $userinfo_response );
+    return {} if ( $h != PE_OK );
+
     return $userinfo_response;
 }
 
+sub _formatValue {
+    my ( $self, $session_value, $type, $array, $attribute, $user ) = @_;
+
+    # If $session_value is not a scalar, return it as is
+    unless ( ref($session_value) ) {
+        if ( defined $session_value ) {
+
+            # Empty strings or lists are invalid values
+            if ( length($session_value) > 0 ) {
+
+                # Format value for JSON output: multi valuation, JSON type...
+                my $separator = $self->conf->{multiValuesSeparator};
+                return $self->_applyType( $session_value, $separator, $type,
+                    $array, $attribute, $user );
+            }
+            else {
+                return undef;
+            }
+        }
+    }
+    return $session_value;
+}
+
+sub _applyType {
+    my ( $self, $session_value, $separator, $type, $array, $attribute, $user )
+      = @_;
+
+    # Array style handling
+    # In auto array mode, split as array only if there are multiple values
+    if ( $array eq "auto" ) {
+        if ( $session_value and $session_value =~ /$separator/ ) {
+            $session_value = [
+                map { $self->_forceType( $_, $type ) }
+                  split( $separator, $session_value )
+            ];
+        }
+        else {
+            $session_value = $self->_forceType( $session_value, $type );
+        }
+
+        # In always array mode, always split (even on empty values)
+    }
+    elsif ( $array eq "always" ) {
+        $session_value = [
+            map { $self->_forceType( $_, $type ) }
+              split( $separator, $session_value )
+        ];
+    }
+
+    # In never array mode, return the string as-is
+    else {
+        # No type coaxing is possible on a flattened string
+        if ( $session_value =~ /$separator/ and $type ne "string" ) {
+            $self->logger->warn( "Cannot force type of value $session_value"
+                  . " for attribute $attribute of user "
+                  . $user
+                  . " because it is multi-valued. "
+                  . "Use auto or always as array type for this attribute" );
+        }
+        else {
+            $session_value = $self->_forceType( $session_value, $type );
+        }
+    }
+
+    return $session_value;
+}
+
+sub _forceType {
+    my ( $self, $val, $type ) = @_;
+
+    # Boolean
+    return ( $val ? JSON::true : JSON::false ) if ( $type eq "bool" );
+
+    # Coax into int
+    return ( $val + 0 ) if ( $type eq "int" );
+
+    # Coax into string
+    return ( $val . "" );
+}
+
 # Return JWT
 # @param payload JWT content
 # @param alg Signature algorithm
@@ -1482,13 +1585,16 @@ sub createJWT {
 # @param rp Internal Relying Party identifier
 # @return String id_token ID Token as JWT
 sub createIDToken {
-    my ( $self, $payload, $rp ) = @_;
+    my ( $self, $req, $payload, $rp ) = @_;
 
     # Get signature algorithm
     my $alg = $self->conf->{oidcRPMetaDataOptions}->{$rp}
       ->{oidcRPMetaDataOptionsIDTokenSignAlg};
     $self->logger->debug("ID Token signature algorithm: $alg");
 
+    my $h = $self->p->processHook( $req, 'oidcGenerateIDToken', $payload, $rp );
+    return undef if ( $h != PE_OK );
+
     return $self->createJWT( $payload, $alg, $rp );
 }
 
@@ -1712,6 +1818,50 @@ sub force_id_claims {
       ->{oidcRPMetaDataOptionsIDTokenForceClaims};
 }
 
+# https://openid.net/specs/openid-connect-core-1_0.html#IDToken
+# Audience(s) that this ID Token is intended for. It MUST contain the OAuth 2.0
+# client_id of the Relying Party as an audience value. It MAY also contain
+# identifiers for other audiences. In the general case, the aud value is an
+# array of case sensitive strings. In the common special case when there is one
+# audience, the aud value MAY be a single case sensitive string.
+sub getAudiences {
+    my ( $self, $rp ) = @_;
+
+    my $client_id = $self->oidcRPList->{$rp}->{oidcRPMetaDataOptionsClientID};
+    my @addAudiences = split /\s+/,
+      ( $self->oidcRPList->{$rp}->{oidcRPMetaDataOptionsAdditionalAudiences}
+          || '' );
+
+    my $result = [$client_id];
+    push @{$result}, @addAudiences;
+
+    return $result;
+}
+
+# Returns the main attribute (sub) to use for this RP
+# It can be a session attribute, or per-RP macro
+sub getUserIDForRP {
+    my ( $self, $req, $rp, $data ) = @_;
+
+    my $user_id_attribute =
+      $self->conf->{oidcRPMetaDataOptions}->{$rp}
+      ->{oidcRPMetaDataOptionsUserIDAttr}
+      || $self->conf->{whatToTrace};
+
+    my $user_id;
+
+    # If the main attribute is a SP macro, resolve it
+    # else, get it directly from session data
+    if ( $self->spMacros->{$rp}->{$user_id_attribute} ) {
+        $user_id =
+          $self->spMacros->{$rp}->{$user_id_attribute}->( $req, $data );
+    }
+    else {
+        $user_id = $data->{$user_id_attribute};
+    }
+    return $user_id;
+}
+
 1;
 
 __END__
