@@ -2,7 +2,7 @@
 # into "plugins" list in lemonldap-ng.ini, section "portal"
 package Lemonldap::NG::Portal::Main::Plugins;
 
-our $VERSION = '2.0.7';
+our $VERSION = '2.0.11';
 
 package Lemonldap::NG::Portal::Main;
 
@@ -14,24 +14,28 @@ use Mouse;
 # Developers: 2FA must be loaded before Notifications
 # Developers: GlobalLogout must be the last loaded plugin
 our @pList = (
-    portalDisplayResetPassword => '::Plugins::MailPasswordReset',
-    portalStatus               => '::Plugins::Status',
-    cda                        => '::Plugins::CDA',
-    notification               => '::Plugins::Notifications',
-    portalCheckLogins          => '::Plugins::History',
-    stayConnected              => '::Plugins::StayConnected',
-    bruteForceProtection       => '::Plugins::BruteForceProtection',
-    grantSessionRules          => '::Plugins::GrantSession',
-    upgradeSession             => '::Plugins::Upgrade',
-    autoSigninRules            => '::Plugins::AutoSignin',
-    checkState                 => '::Plugins::CheckState',
-    portalForceAuthn           => '::Plugins::ForceAuthn',
-    checkUser                  => '::Plugins::CheckUser',
-    impersonationRule          => '::Plugins::Impersonation',
-    contextSwitchingRule       => '::Plugins::ContextSwitching',
-    decryptValueRule           => '::Plugins::DecryptValue',
-    globalLogoutRule           => '::Plugins::GlobalLogout',
-    refreshSessions            => '::Plugins::Refresh',
+    portalDisplayResetPassword          => '::Plugins::MailPasswordReset',
+    portalDisplayCertificateResetByMail => '::Plugins::CertificateResetByMail',
+    portalStatus                        => '::Plugins::Status',
+    cda                                 => '::Plugins::CDA',
+    notification                        => '::Plugins::Notifications',
+    stayConnected                       => '::Plugins::StayConnected',
+    portalCheckLogins                   => '::Plugins::History',
+    bruteForceProtection                => '::Plugins::BruteForceProtection',
+    grantSessionRules                   => '::Plugins::GrantSession',
+    upgradeSession                      => '::Plugins::Upgrade',
+    autoSigninRules                     => '::Plugins::AutoSignin',
+    checkState                          => '::Plugins::CheckState',
+    portalForceAuthn                    => '::Plugins::ForceAuthn',
+    checkUser                           => '::Plugins::CheckUser',
+    impersonationRule                   => '::Plugins::Impersonation',
+    contextSwitchingRule                => '::Plugins::ContextSwitching',
+    decryptValueRule                    => '::Plugins::DecryptValue',
+    findUser                            => '::Plugins::FindUser',
+    adaptativeAuthenticationLevelRules =>
+      '::Plugins::AdaptativeAuthenticationLevel',
+    globalLogoutRule => '::Plugins::GlobalLogout',
+    refreshSessions  => '::Plugins::Refresh',
 );
 
 ##@method list enabledPlugins
@@ -50,11 +54,6 @@ sub enabledPlugins {
         }
     }
 
-    # Load static plugin list
-    for ( my $i = 0 ; $i < @pList ; $i += 2 ) {
-        push @res, $pList[ $i + 1 ] if ( $conf->{ $pList[$i] } );
-    }
-
     # Load single session
     push @res, '::Plugins::SingleSession'
       if ( $conf->{singleSession}
@@ -62,6 +61,19 @@ sub enabledPlugins {
         or $conf->{singleUserByIP}
         or $conf->{notifyOther} );
 
+    # Load static plugin list
+    for ( my $i = 0 ; $i < @pList ; $i += 2 ) {
+        my $pluginConf = $conf->{ $pList[$i] };
+        if ( ref($pluginConf) eq "HASH" ) {
+
+            # Do not load plugin if config is an empty hash
+            push @res, $pList[ $i + 1 ] if %{$pluginConf};
+        }
+        else {
+            push @res, $pList[ $i + 1 ] if $pluginConf;
+        }
+    }
+
     # Check if SOAP is enabled
     push @res, '::Plugins::SOAPServer'
       if ( $conf->{soapSessionServer}
@@ -77,8 +89,6 @@ sub enabledPlugins {
     # Check if register is enabled
     push @res, '::Plugins::Register'
       if ( $conf->{registerDB} and $conf->{registerDB} ne 'Null' );
-    push @res, '::Plugins::CertificateResetByMail'
-      if ( $conf->{portalDisplayCertificateResetByMail});
 
     # Check if custom plugins are required
     # TODO: change this name
