@@ -5,27 +5,40 @@ use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants qw(
   PE_OK
-  PE_BADOLDPASSWORD
   PE_PASSWORD_OK
+  PE_BADOLDPASSWORD
   PE_PASSWORD_MISMATCH
-  PE_PP_MUST_SUPPLY_OLD_PASSWORD
   PE_PP_PASSWORD_TOO_SHORT
+  PE_PP_NOT_ALLOWED_CHARACTER
+  PE_PP_NOT_ALLOWED_CHARACTERS
+  PE_PP_MUST_SUPPLY_OLD_PASSWORD
   PE_PP_INSUFFICIENT_PASSWORD_QUALITY
 );
 
 extends 'Lemonldap::NG::Portal::Main::Plugin';
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.10';
 
 # INITIALIZATION
 
+has requireOldPwdRule => ( is => 'rw' );
+
 sub init {
-    $_[0]->p->{_passwordDB} = $_[0];
+    my ($self) = shift;
+    $self->requireOldPwdRule(
+        $self->p->buildRule(
+            $self->conf->{portalRequireOldPassword},
+            'portalRequireOldPassword'
+        )
+    );
+    return 0 unless $self->requireOldPwdRule;
+
+    $self->p->{_passwordDB} = $self;
 }
 
 # INTERFACE
 
-sub forAuthUser { '_modifyPassword' }
+use constant forAuthUser => '_modifyPassword';
 
 # RUNNING METHODS
 
@@ -40,8 +53,26 @@ sub _modifyPassword {
     return PE_PASSWORD_MISMATCH
       unless ( $req->data->{newpassword} eq $req->param('confirmpassword') );
 
+    my $oldPwdRule = $self->p->HANDLER->buildSub(
+        $self->p->HANDLER->substitute(
+            $self->conf->{portalRequireOldPassword}
+        )
+    );
+    unless ($oldPwdRule) {
+        my $error = $self->p->HANDLER->tsv->{jail}->error || '???';
+    }
+
+    my $pwdPolicyRule = $self->p->HANDLER->buildSub(
+        $self->p->HANDLER->substitute(
+            $self->conf->{passwordPolicyActivation}
+        )
+    );
+    unless ($pwdPolicyRule) {
+        my $error = $self->p->HANDLER->tsv->{jail}->error || '???';
+    }
+
     # Check if portal require old password
-    if ( $self->conf->{portalRequireOldPassword} or $requireOldPwd ) {
+    if ( $oldPwdRule->( $req, $req->userData ) or $requireOldPwd ) {
 
         # TODO: verify oldpassword
         unless ( $req->data->{oldpassword} = $req->param('oldpassword') ) {
@@ -54,13 +85,20 @@ sub _modifyPassword {
           unless ( $self->confirm( $req, $req->data->{oldpassword} ) );
     }
 
-    my $cpq = $self->checkPasswordQuality( $req->data->{newpassword} );
+    my $cpq =
+        $pwdPolicyRule->( $req, $req->userData )
+      ? $self->checkPasswordQuality( $req->data->{newpassword} )
+      : PE_OK;
     return $cpq unless ( $cpq == PE_OK );
 
     # Call password package
     my $res = $self->modifyPassword( $req, $req->data->{newpassword} );
     if ( $res == PE_PASSWORD_OK ) {
         $self->logger->debug( 'Update password in session for ' . $req->user );
+        my $userlog = $req->sessionInfo->{ $self->conf->{whatToTrace} };
+        my $iplog   = $req->sessionInfo->{ipAddr};
+        $self->userLogger->notice("Password changed for $userlog ($iplog)")
+          if ( defined $userlog and $iplog );
         my $infos;
 
         # Store new password if asked
@@ -69,7 +107,7 @@ sub _modifyPassword {
                 $req,
                 {
                     _passwordDB => $self->p->getModule( $req, 'password' ),
-                    _password   => $req->{newpassword}
+                    _password   => $req->data->{newpassword}
                 }
             );
         }
@@ -131,6 +169,42 @@ sub checkPasswordQuality {
         }
     }
 
+    ### Special characters policy
+    my $speChars = $self->conf->{passwordPolicySpecialChar};
+    $speChars =~ s/\s+//g;
+
+    ## Min special characters
+    # Just number of special characters must be checked
+    if ( $self->conf->{passwordPolicyMinSpeChar} && $speChars eq '__ALL__' ) {
+        my $spe = $password =~ s/\w//g;
+        if ( $spe < $self->conf->{passwordPolicyMinSpeChar} ) {
+            $self->logger->error("Password has not enough special characters");
+            return PE_PP_INSUFFICIENT_PASSWORD_QUALITY;
+        }
+        return PE_OK;
+    }
+
+    # Number of special characters must be checked
+    if ( $self->conf->{passwordPolicyMinSpeChar} && $speChars ) {
+        my $test = $password;
+        my $spe  = $test =~ s/[\Q$speChars\E]//g;
+        if ( $spe < $self->conf->{passwordPolicyMinSpeChar} ) {
+            $self->logger->error("Password has not enough special characters");
+            return PE_PP_INSUFFICIENT_PASSWORD_QUALITY;
+        }
+    }
+
+    ## Fobidden special characters
+    $password =~ s/[\Q$speChars\E\w]//g;
+    if ($password) {
+        $self->logger->error( 'Password contains '
+              . length($password)
+              . " forbidden character(s): $password" );
+        return length($password) > 1
+          ? PE_PP_NOT_ALLOWED_CHARACTERS
+          : PE_PP_NOT_ALLOWED_CHARACTER;
+    }
+
     return PE_OK;
 }
 
