@@ -3,22 +3,23 @@ package Lemonldap::NG::Portal::Password::LDAP;
 use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants qw(
-  PE_PASSWORD_OK
-  PE_LDAPERROR
-  PE_LDAPCONNECTFAILED
   PE_ERROR
+  PE_LDAPERROR
+  PE_PASSWORD_OK
+  PE_LDAPCONNECTFAILED
 );
 
-extends 'Lemonldap::NG::Portal::Lib::LDAP',
-  'Lemonldap::NG::Portal::Password::Base';
+extends qw(
+  Lemonldap::NG::Portal::Lib::LDAP
+  Lemonldap::NG::Portal::Password::Base
+);
 
-our $VERSION = '2.0.2';
+our $VERSION = '2.0.10';
 
 sub init {
     my ($self) = @_;
-    $self->ldap
-      and $self->filter
-      and $self->Lemonldap::NG::Portal::Password::Base::init;
+    return (  $self->Lemonldap::NG::Portal::Password::Base::init
+          and $self->Lemonldap::NG::Portal::Lib::LDAP::init );
 }
 
 # Confirmation is done by Lib::Net::LDAP::userModifyPassword
@@ -27,20 +28,30 @@ sub confirm {
 }
 
 sub modifyPassword {
-    my ( $self, $req, $pwd ) = @_;
+    my ( $self, $req, $pwd, $useMail ) = @_;
     my $dn;
-    if ( $req->userData->{_dn} ) {
-        $dn = $req->userData->{_dn};
+    my $requireOldPassword;
+
+    # If the password change is done in a different backend,
+    # we need to reload the correct DN
+    $self->getUser( $req, useMail => $useMail )
+      if $self->conf->{ldapGetUserBeforePasswordChange};
+
+    if ( $req->data->{dn} ) {
+        $dn                 = $req->data->{dn};
+        $requireOldPassword = $self->requireOldPwdRule->( $req, $req->userData );
         $self->logger->debug("Get DN from request data: $dn");
     }
     else {
-        $dn = $req->sessionInfo->{_dn};
+        $dn                 = $req->sessionInfo->{_dn};
+        $requireOldPassword = $self->requireOldPwdRule->( $req, $req->sessionInfo );
         $self->logger->debug("Get DN from session data: $dn");
     }
     unless ($dn) {
         $self->logger->error('"dn" is not set, aborting password modification');
         return PE_ERROR;
     }
+    $requireOldPassword = 0 if $useMail;
 
     # Ensure connection is valid
     $self->bind;
@@ -48,7 +59,8 @@ sub modifyPassword {
 
     # Call the modify password method
     my $code =
-      $self->ldap->userModifyPassword( $dn, $pwd, $req->data->{oldpassword} );
+      $self->ldap->userModifyPassword( $dn, $pwd, $req->data->{oldpassword},
+        0, $requireOldPassword );
 
     unless ( $code == PE_PASSWORD_OK ) {
         return $code;
