@@ -4,10 +4,11 @@ use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants qw(
   PE_APACHESESSIONERROR
+  PE_ERROR
   PE_OK
 );
 
-our $VERSION = '2.0.3';
+our $VERSION = '2.0.8';
 
 extends 'Lemonldap::NG::Common::Module';
 
@@ -37,9 +38,18 @@ sub changeUrldc {
             $cdaInfos->{cookie_name}  = $self->{conf}->{cookieName};
         }
         else {
-            $cdaInfos->{cookie_value} =
-              $req->{sessionInfo}->{_httpSession};
-            $cdaInfos->{cookie_name} = $self->{conf}->{cookieName} . "http";
+            if ( $req->{sessionInfo}->{_httpSession} ) {
+                $cdaInfos->{cookie_value} =
+                  $req->{sessionInfo}->{_httpSession};
+                $cdaInfos->{cookie_name} = $self->{conf}->{cookieName} . "http";
+            }
+            else {
+                $self->logger->error(
+                        "Session does not contain _httpSession field. "
+                      . "Portal must be accessed over HTTPS when using CDA with double cookie"
+                );
+                return PE_ERROR;
+            }
         }
 
         my $cdaSession =
@@ -60,7 +70,7 @@ sub changeUrldc {
 
         $self->logger->debug( "CDA redirection to " . $req->{urldc} );
     }
-    PE_OK;
+    return PE_OK;
 }
 
 1;
