@@ -11,31 +11,31 @@ use POSIX qw(strftime);
 use Lemonldap::NG::Common::FormEncode;
 use Lemonldap::NG::Portal::Main::Constants qw(
   PE_OK
-  PE_BADCREDENTIALS
+  PE_MAILOK
+  PE_NOTOKEN
+  PE_MAILERROR
   PE_BADMAILTOKEN
   PE_CAPTCHAEMPTY
   PE_CAPTCHAERROR
-  PE_MAILCONFIRMATION_ALREADY_SENT
-  PE_MAILCONFIRMOK
-  PE_MAILERROR
-  PE_MAILFIRSTACCESS
-  PE_MAILFORMEMPTY
   PE_MAILNOTFOUND
-  PE_MAILOK
-  PE_MALFORMEDUSER
-  PE_NOTOKEN
   PE_TOKENEXPIRED
   PE_USERNOTFOUND
+  PE_MAILCONFIRMOK
+  PE_MAILFORMEMPTY
+  PE_MALFORMEDUSER
+  PE_BADCREDENTIALS
+  PE_MAILFIRSTACCESS
   PE_RESETCERTIFICATE_INVALID
   PE_RESETCERTIFICATE_FORMEMPTY
   PE_RESETCERTIFICATE_FIRSTACCESS
+  PE_MAILCONFIRMATION_ALREADY_SENT
 );
 
-our $VERSION = '2.0.8';
+our $VERSION = '2.0.10';
 
 extends qw(
-  Lemonldap::NG::Portal::Main::Plugin
   Lemonldap::NG::Portal::Lib::SMTP
+  Lemonldap::NG::Portal::Main::Plugin
   Lemonldap::NG::Portal::Lib::_tokenRule
 );
 
@@ -234,7 +234,7 @@ sub _certificateReset {
     );
     if ( my $error = $self->p->process( $req, useMail => $searchByMail ) ) {
         if ( $error == PE_USERNOTFOUND or $error == PE_BADCREDENTIALS ) {
-            $self->userLogger->warn( 'Reset asked for an unvalid user ('
+            $self->userLogger->warn( 'Reset asked for an invalid user ('
                   . $req->param('mail')
                   . ')' );
 
