@@ -6,26 +6,29 @@ use Mouse;
 use POSIX qw(strftime);
 use Lemonldap::NG::Common::FormEncode;
 use Lemonldap::NG::Portal::Main::Constants qw(
+  PE_OK
+  PE_MAILOK
+  PE_NOTOKEN
+  PE_MAILERROR
   PE_BADMAILTOKEN
   PE_CAPTCHAEMPTY
   PE_CAPTCHAERROR
-  PE_MALFORMEDUSER
-  PE_MAILCONFIRMATION_ALREADY_SENT
-  PE_MAILCONFIRMOK
-  PE_MAILERROR
-  PE_MAILOK
-  PE_NOTOKEN
-  PE_OK
-  PE_REGISTERALREADYEXISTS
-  PE_REGISTERFIRSTACCESS
-  PE_REGISTERFORMEMPTY
   PE_TOKENEXPIRED
+  PE_MAILCONFIRMOK
+  PE_MALFORMEDUSER
+  PE_REGISTERFORMEMPTY
+  PE_REGISTERFIRSTACCESS
+  PE_REGISTERALREADYEXISTS
+  PE_MAILCONFIRMATION_ALREADY_SENT
 );
 
-our $VERSION = '2.0.2';
+our $VERSION = '2.0.10';
 
-extends 'Lemonldap::NG::Portal::Main::Plugin',
-  'Lemonldap::NG::Portal::Lib::SMTP';
+extends qw(
+  Lemonldap::NG::Portal::Lib::SMTP
+  Lemonldap::NG::Portal::Main::Plugin
+  Lemonldap::NG::Portal::Lib::_tokenRule
+);
 
 # PROPERTIES
 
@@ -77,7 +80,7 @@ sub init {
     }
 
     # Initialize form token if needed (captcha provides also a token)
-    elsif ( $self->conf->{requireToken} ) {
+    else {
         $_[0]->ott(
             $_[0]->p->loadModule('Lemonldap::NG::Portal::Lib::OneTimeToken') )
           or return 0;
@@ -165,9 +168,10 @@ sub _register {
             and
             !$self->getRegisterSession( $req->data->{registerInfo}->{mail} ) )
         {
+
             # Check if token exists
             my $token;
-            if ( $self->ott or $self->captcha ) {
+            if ( $self->ottRule->( $req, {} ) or $self->captcha ) {
                 $token = $req->param('token');
                 unless ($token) {
                     $self->setSecurity($req);
@@ -199,7 +203,7 @@ sub _register {
                 }
                 $self->logger->debug("Captcha code verified");
             }
-            elsif ( $self->ott ) {
+            elsif ( $self->ottRule->( $req, {} ) ) {
                 unless ( $self->ott->getToken($token) ) {
                     $self->setSecurity($req);
                     $self->userLogger->notice(
@@ -308,7 +312,8 @@ sub _register {
         my $html = 1;
 
         # Use HTML template
-        $body = $self->loadTemplate(
+        $body = $self->loadMailTemplate(
+            $req,
             'mail_register_confirm',
             filter => $tr,
             params => \%tplPrms
@@ -367,7 +372,8 @@ sub _register {
     my $html = 1;
 
     # Use HTML template
-    $body = $self->loadTemplate(
+    $body = $self->loadMailTemplate(
+        $req,
         'mail_register_done',
         filter => $tr,
         params => \%tplPrms
@@ -452,7 +458,7 @@ sub display {
 
     # Display captcha if it's enabled
     if ( $req->captcha ) {
-        $templateParams{CAPTCHA_SRC} = $req->captcha;
+        $templateParams{CAPTCHA_SRC}  = $req->captcha;
         $templateParams{CAPTCHA_SIZE} = $self->conf->{captcha_size} || 6;
     }
     if ( $req->token ) {
@@ -530,7 +536,7 @@ sub setSecurity {
     if ( $self->captcha ) {
         $self->captcha->setCaptcha($req);
     }
-    elsif ( $self->ott ) {
+    elsif ( $self->ottRule->( $req, {} ) ) {
         $self->ott->setToken($req);
     }
 }
