@@ -2,7 +2,6 @@
 
 package Lemonldap::NG::Portal::Plugins::StayConnected;
 
-use 5.16.0;
 use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants qw(
@@ -10,14 +9,14 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_SENDRESPONSE
 );
 
-our $VERSION = '2.0.7';
+our $VERSION = '2.0.10';
 
 extends 'Lemonldap::NG::Portal::Main::Plugin';
 
 # INTERFACE
 
-use constant endAuth => 'newDevice';
-use constant beforeAuth => 'check';
+use constant endAuth      => 'newDevice';
+use constant beforeAuth   => 'check';
 use constant beforeLogout => 'logout';
 
 # INITIALIZATION
@@ -32,32 +31,48 @@ has ott => (
         return $ott;
     }
 );
+has cookieName => (
+    is      => 'rw',
+    lazy    => 1,
+    default => sub {
+        $_[0]->{conf}->{stayConnectedCookieName} || 'llngconnection';
+    }
+);
 
 # Default timeout: 1 month
 has timeout => (
     is      => 'rw',
     lazy    => 1,
     default => sub {
-        $_[0]->{conf}->{stayConnectedTimeout} || 2678400;
+        $_[0]->{conf}->{stayConnectedTimeout} || 2592000;
     }
 );
 
 sub init {
     my ($self) = @_;
     $self->addAuthRoute( registerbrowser => 'storeBrowser', ['POST'] );
+
     return 1;
 }
 
 # RUNNING METHODS
 
-# Registration: detect if user wants to stay connected. Then ask for
-#               fingerprint
+# Registration: detect if user wants to stay connected.
+# Then ask for browser fingerprint
 sub newDevice {
     my ( $self, $req ) = @_;
 
+    my $checkLogins = $req->param('checkLogins');
+    $self->logger->debug("StayConnected: checkLogins set") if $checkLogins;
+
     if ( $req->param('stayconnected') ) {
         my $token = $self->ott->createToken( {
-                name => $req->sessionInfo->{ $self->conf->{whatToTrace} }
+                name => $req->sessionInfo->{ $self->conf->{whatToTrace} },
+                (
+                    $checkLogins
+                    ? ( history => $req->sessionInfo->{_loginHistory} )
+                    : ()
+                )
             }
         );
         $req->response(
@@ -65,9 +80,10 @@ sub newDevice {
                 $req,
                 '../common/registerBrowser',
                 params => {
-                    URL    => $req->urldc,
-                    TOKEN  => $token,
-                    ACTION => '/registerbrowser',
+                    URL         => $req->urldc,
+                    TOKEN       => $token,
+                    ACTION      => '/registerbrowser',
+                    CHECKLOGINS => $checkLogins
                 }
             )
         );
@@ -92,7 +108,7 @@ sub storeBrowser {
                           $self->conf->{globalStorageOptions},
                         kind => "SSO",
                         info => {
-                            _utime          => time + $self->timeout,
+                            _utime          => time + $self->timeout(),
                             _session_uid    => $uid,
                             _connectedSince => time,
                             dataKeep        => $req->data->{dataToKeep},
@@ -103,11 +119,14 @@ sub storeBrowser {
                     # Cookie available 30 days
                     $req->addCookie(
                         $self->p->cookie(
-                            name    => 'llngconnexion',
+                            name    => $self->cookieName(),
                             value   => $ps->id,
-                            max_age => 2592000,
+                            max_age => $self->timeout(),
+                            secure  => $self->conf->{securedCookie},
                         )
                     );
+                    $req->sessionInfo->{_loginHistory} = $tmp->{history}
+                      if exists $tmp->{history};
                 }
                 else {
                     $self->logger->warn("Browser hasn't return fingerprint");
@@ -119,15 +138,16 @@ sub storeBrowser {
             }
         }
         else {
-            $self->userLogger->error("StayConnected called with an expired token");
+            $self->userLogger->error(
+                "StayConnected called with an expired token");
         }
     }
     else {
         $self->userLogger->error('StayConnected called without token');
     }
 
-    # Deliver cookie llngbrowser
-    return $self->p->do( $req, [ sub { PE_OK } ] );
+    # Return persistent connection cookie
+    return $self->p->do( $req, [ @{ $self->p->endAuth }, sub { PE_OK } ] );
 }
 
 # Check for:
@@ -137,14 +157,17 @@ sub storeBrowser {
 # Then delete authentication methods from "steps" array.
 sub check {
     my ( $self, $req ) = @_;
-    if ( my $cid = $req->cookies->{llngconnexion} ) {
+    if ( my $cid = $req->cookies->{ $self->cookieName() } ) {
         my $ps = Lemonldap::NG::Common::Session->new(
             storageModule        => $self->conf->{globalStorage},
             storageModuleOptions => $self->conf->{globalStorageOptions},
             kind                 => "SSO",
             id                   => $cid,
         );
-        if ( $ps and my $uid = $ps->data->{_session_uid} ) {
+        if (    $ps
+            and my $uid = $ps->data->{_session_uid}
+            and time() < $ps->data->{_utime} )
+        {
             $self->logger->debug('Persistent connection found');
             if (    my $fg = $req->param('fg')
                 and my $token = $req->param('token') )
@@ -166,6 +189,8 @@ sub check {
                     }
                     else {
                         $self->userLogger->warn("Fingerprint changed for $uid");
+                        $ps->remove;
+                        $self->logout($req);
                     }
                 }
                 else {
@@ -190,6 +215,13 @@ sub check {
         }
         else {
             $self->userLogger->notice('Persistent connection expired');
+            unless ( $ps->{error} ) {
+                $self->logger->debug(
+                    'Persistent connection session id = ' . $ps->{id} );
+                $self->logger->debug( 'Persistent connection session _utime = '
+                      . $ps->data->{_utime} );
+                $ps->remove;
+            }
         }
     }
     return PE_OK;
@@ -199,9 +231,10 @@ sub logout {
     my ( $self, $req ) = @_;
     $req->addCookie(
         $self->p->cookie(
-            name    => 'llngconnexion',
+            name    => $self->cookieName(),
             value   => 0,
-            expires => 'Wed, 21 Oct 2015 00:00:00 GMT'
+            expires => 'Wed, 21 Oct 2015 00:00:00 GMT',
+            secure  => $self->conf->{securedCookie},
         )
     );
     return PE_OK;
