@@ -4,7 +4,7 @@ use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants qw(PE_OK);
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.11';
 
 extends 'Lemonldap::NG::Common::Module', 'Lemonldap::NG::Portal::Lib::Remote';
 
@@ -12,6 +12,12 @@ extends 'Lemonldap::NG::Common::Module', 'Lemonldap::NG::Portal::Lib::Remote';
 
 *getUser = *Lemonldap::NG::Portal::Lib::Remote::checkRemoteId;
 
+sub findUser {
+
+    # Nothing to do here
+    PE_OK;
+}
+
 sub setSessionInfo {
     my ( $self, $req ) = @_;
     delete $req->data->{rSessionInfo}->{_session_id};
