@@ -57,7 +57,9 @@ use strict;
 use Data::Dumper;
 use File::Find;
 use LWP::UserAgent;
+use Time::Fake;
 use URI::Escape;
+use MIME::Base64;
 use Lemonldap::NG::Common::FormEncode;
 
 #use 5.10.0;
@@ -65,7 +67,6 @@ use Lemonldap::NG::Common::FormEncode;
 no warnings 'redefine';
 
 BEGIN {
-    require 't/Time-Fake.pm';
     use_ok('Lemonldap::NG::Portal::Main');
 }
 
@@ -76,11 +77,13 @@ $Data::Dumper::Useperl  = 1;
 my $ini;
 
 use File::Temp 'tempfile', 'tempdir';
+use File::Copy 'copy';
 our $tmpDir = $LLNG::TMPDIR
   || tempdir( 'tmpSessionXXXXX', DIR => 't/sessions', CLEANUP => 1 );
 mkdir "$tmpDir/lock";
 mkdir "$tmpDir/saml";
 mkdir "$tmpDir/saml/lock";
+copy( "t/lmConf-1.json", "$tmpDir/lmConf-1.json" );
 
 =head4 count($inc)
 
@@ -94,6 +97,29 @@ sub count {
     return $count;
 }
 
+=head4 buildForm($params)
+
+Convenience method that builds a url-encoded query string from a hash of arguments
+
+=cut
+
+sub buildForm {
+    my $fields = shift;
+    my $query  = join(
+        '&',
+        map {
+            "$_="
+              . (
+                $fields->{$_}
+                ? uri_escape( uri_unescape( $fields->{$_} ) )
+                : ''
+              )
+          }
+          keys(%$fields)
+    );
+    return $query;
+}
+
 =head4 explain( $result, $expected_result )
 
 Used to display error if test fails:
@@ -128,21 +154,29 @@ sub clean_sessions {
 }
 
 sub count_sessions {
-    my $dir = shift;
-    $dir ||= $tmpDir;
+    my ( $kind, $dir ) = @_;
     my $nbr = 0;
+    $kind ||= 'SSO';
+    $dir  ||= $tmpDir;
 
     opendir D, $dir or die $!;
     foreach ( grep { /^\w{64}$/ } readdir(D) ) {
-        $nbr++;
+        open( my $fh, '<', "$dir/$_" ) or die($!);
+        while (<$fh>) {
+            chomp;
+            if ( $_ =~ /"_session_kind":"$kind"/ ) {
+                $nbr++;
+                last;
+            }
+        }
+        close $fh;
     }
     $nbr;
 }
 
 sub getCache {
     require Cache::FileCache;
-    return Cache::FileCache->new(
-        {
+    return Cache::FileCache->new( {
             namespace   => 'lemonldap-ng-session',
             cache_root  => $tmpDir,
             cache_depth => 0,
@@ -275,18 +309,7 @@ m@<form.+?action="(?:(?:http://([^/]+))?(/.*?)?|(#))".+method="(post|get)"@is,
               m#<input.+?name="([^"]+)"[^>]+(?:value="([^"]*?)")?#gs,
             %fields
         );
-        my $query = join(
-            '&',
-            map {
-                "$_="
-                  . (
-                    $fields{$_}
-                    ? uri_escape( uri_unescape( $fields{$_} ) )
-                    : ''
-                  )
-              }
-              keys(%fields)
-        );
+        my $query = buildForm( \%fields );
         foreach my $f (@requiredFields) {
             ok( exists $fields{$f}, qq{ Field "$f" is defined} );
             count(1);
@@ -358,10 +381,22 @@ sub expectBadRequest {
     count(1);
 }
 
-=head4 expectReject( $res, $code )
+=head4 expectPortalError( $res, $errnum )
 
-Verify that returned code is 401 and JSON result contains C<error:"$code">.
-Note that it works only for Ajax request (see below).
+Verify that an error is displayed on the portal
+=cut
+
+sub expectPortalError {
+    my ( $res, $errnum, $message ) = @_;
+    $errnum ||= 9;
+    like( $res->[2]->[0], qr/<span trmsg="$errnum">/, $message );
+    count(1);
+}
+
+=head4 expectReject( $res, $status, $code )
+
+Verify that returned code is 401 (or $status)  and JSON result contains
+C<error:"$code">.  Note that it works only for Ajax request (see below).
 
 =cut
 
@@ -501,6 +536,51 @@ sub tempdb {
     return "$tmpDir/userdb.db";
 }
 
+my %handlerOR;
+
+=head4 register
+
+Registers a new LLNG instance
+
+=cut
+
+sub register {
+    my ( $type, $constructor ) = @_;
+    my $obj;
+    @Lemonldap::NG::Handler::Main::_onReload = ();
+    &Lemonldap::NG::Handler::Main::cfgNum( 0, 0 );
+    ok( $obj = $constructor->(), 'Register $type' );
+    count(1);
+    $handlerOR{$type} = \@Lemonldap::NG::Handler::Main::_onReload;
+    return $obj;
+}
+
+=head4 register
+
+Switch to a registered instance
+
+=cut
+
+sub switch {
+    my $type = shift;
+    return [] unless $handlerOR{$type};
+    note( '==> Switching to ' . uc($type) . ' <==' );
+    @Lemonldap::NG::Handler::Main::_onReload = @{
+        $handlerOR{$type};
+    };
+}
+
+=head4 encodeUrl( $url );
+
+Encode URL like the handler would, see ::Handler::Main
+
+=cut
+
+sub encodeUrl {
+    my ($url) = @_;
+    return uri_escape( encode_base64( $url, '' ) );
+}
+
 =head2 LLNG::Manager::Test Class
 
 =cut
@@ -515,7 +595,7 @@ extends 'Lemonldap::NG::Common::PSGI::Cli::Lib';
 our $defaultIni = {
     configStorage => {
         type    => 'File',
-        dirName => 't',
+        dirName => "$tmpDir",
     },
     localSessionStorage        => 'Cache::FileCache',
     localSessionStorageOptions => {
@@ -523,12 +603,21 @@ our $defaultIni = {
         cache_root  => $tmpDir,
         cache_depth => 0,
     },
-    logLevel             => 'error',
-    cookieName           => 'lemonldap',
-    domain               => 'example.com',
-    templateDir          => 'site/templates',
-    staticPrefix         => '/static',
-    securedCookie        => 0,
+    logLevel              => 'error',
+    cookieName            => 'lemonldap',
+    domain                => 'example.com',
+    templateDir           => 'site/templates',
+    staticPrefix          => '/static',
+    tokenUseGlobalStorage => 0,
+    securedCookie         => 0,
+
+    # If lasso is too old, force algo to SHA1
+    (
+        eval
+'use Lasso; Lasso::check_version( 2, 5, 1, Lasso::Constants::CHECK_VERSION_NUMERIC) ? 0:1'
+        ? ( samlServiceSignatureMethod => "RSA_SHA1" )
+        : ()
+    ),
     https                => 0,
     globalStorageOptions => {
         Directory     => $tmpDir,
@@ -600,7 +689,7 @@ has ini => (
         main::ok( $self->{p} = $self->class->new(), 'Portal object' );
         main::count(1);
         unless ( $self->confFailure ) {
-            main::ok( $self->{p}->init($ini), 'Init' );
+            main::ok( $self->{p}->init($ini),           'Init' );
             main::ok( $self->{app} = $self->{p}->run(), 'Portal app' );
             main::count(2);
             no warnings 'redefine';
@@ -610,17 +699,35 @@ has ini => (
                 uid  => 'french',
                 cn   => 'Frédéric Accents',
                 mail => 'fa@badwolf.org',
-            };
-            $Lemonldap::NG::Portal::UserDB::Demo::demoAccounts{davros} = {
-                uid  => 'davros',
-                cn   => 'Bad Guy',
-                mail => 'davros@badguy.org',
+                guy  => '',
+                type => '',
             };
             $Lemonldap::NG::Portal::UserDB::Demo::demoAccounts{russian} = {
                 uid  => 'russian',
                 cn   => 'Русский',
                 mail => 'ru@badwolf.org',
+                guy  => '',
+                type => '',
             };
+            $Lemonldap::NG::Portal::UserDB::Demo::demoAccounts{davros} = {
+                uid  => 'davros',
+                cn   => 'Bad Guy',
+                mail => 'davros@badguy.org',
+                guy  => 'bad',
+                type => 'character',
+            };
+            $Lemonldap::NG::Portal::UserDB::Demo::demoAccounts{dalek} = {
+                uid  => 'dalek',
+                cn   => 'The Daleks',
+                mail => 'dalek@badguy.org',
+                guy  => 'bad',
+                type => 'mutant',
+            };
+            push
+              @{ $Lemonldap::NG::Portal::UserDB::Demo::demoGroups{earthlings} },
+              "french", "russian";
+            push @{ $Lemonldap::NG::Portal::UserDB::Demo::demoGroups{users} },
+              "french", "russian", "davros";
         }
         $self;
     }
@@ -705,8 +812,7 @@ to test content I<(to launch a C<expectForm()> for example)>.
 
 sub _get {
     my ( $self, $path, %args ) = @_;
-    my $res = $self->app->(
-        {
+    my $res = $self->app->( {
             'HTTP_ACCEPT' => $args{accept}
               || 'application/json, text/plain, */*',
             'HTTP_ACCEPT_LANGUAGE' => 'fr,fr-FR;q=0.8,en-US;q=0.5,en;q=0.3',
@@ -732,6 +838,7 @@ sub _get {
             'SERVER_NAME'     => 'auth.example.com',
             'SERVER_PORT'     => '80',
             'SERVER_PROTOCOL' => 'HTTP/1.1',
+            'psgi.url_scheme' => ( $args{secure} ? 'https' : 'http' ),
             ( $args{custom} ? %{ $args{custom} } : () ),
         }
     );
@@ -758,8 +865,7 @@ sub _post {
     my ( $self, $path, $body, %args ) = @_;
     die "$body must be a IO::Handle"
       unless ( ref($body) and $body->can('read') );
-    my $res = $self->app->(
-        {
+    my $res = $self->app->( {
             'HTTP_ACCEPT' => $args{accept}
               || 'application/json, text/plain, */*',
             'HTTP_ACCEPT_LANGUAGE' => 'fr,fr-FR;q=0.8,en-US;q=0.5,en;q=0.3',
@@ -771,10 +877,12 @@ sub _post {
             'PATH_INFO' => $path,
             ( $args{query}   ? ( QUERY_STRING => $args{query} )   : () ),
             ( $args{referer} ? ( REFERER      => $args{referer} ) : () ),
-            'REMOTE_ADDR' => '127.0.0.1',
             (
-                $args{remote_user}
-                ? ( 'REMOTE_USER' => $args{remote_user} )
+                $args{ip} ? ( 'REMOTE_ADDR' => $args{ip} )
+                : ( 'REMOTE_ADDR' => '127.0.0.1' )
+            ),
+            (
+                $args{remote_user} ? ( 'REMOTE_USER' => $args{remote_user} )
                 : ()
             ),
             'REQUEST_METHOD' => $args{method} || 'POST',
@@ -783,6 +891,7 @@ sub _post {
             'SERVER_NAME'    => 'auth.example.com',
             'SERVER_PORT'    => '80',
             'SERVER_PROTOCOL' => 'HTTP/1.1',
+            'psgi.url_scheme' => ( $args{secure} ? 'https' : 'http' ),
             ( $args{custom} ? %{ $args{custom} } : () ),
             'psgix.input.buffered' => 0,
             'psgi.input'           => $body,
